/* 
* ORBITER vessel module: CVELtitan
* by Erik Anderson (sputnik)
* erik_h_anderson@hotmail.com
*
* Autopilot by rjcroy
*  (Pitch axis only)
*
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
* 
*
* Requires CVEL library --                             
* please see ProjectGanymede on SourceForge for the latest source *
*
*/

#include "cveltitan.h"
#include "../../include/orbitersdk.h"
#include "../../include/cvel.h"
#include "../../include/cvelmessage.h"
#include "../../include/guidance.h"
//#include "../../include/orbitersoundsdk3.h"
//#include "multi_stage2.h"
#include "stdio.h"
#include "math.h"

																					
void CVELTITAN::InitialiseGuidance()
{
	

	bManualSeparate = false;
	stage_sep=0;
	ignition= false;
//	time=0.0;

	pitchinput = 0.0;
	rollinput = 0.0;
	engineignited = false;
	last_simt=0.0;
	ignition_time=0.0;
	first_guidance_pass = true;

	// rjcroy Guidance variables

	INCLINATION = 51.63 ;		// degrees

	guidance = false;			// guidance switch
	Alt_Flag = false;			// True if max launch altitude reached
	MECO_Cond = false;			// True if MECO conditions met
	First_Guidance_Pass = false;
	T_minus_10 = true;			// True at T minus 10 seconds.
	TZero = -100;				// Time Zero - Launch time
	PC_D = 0.9;					// Pitch control coefficient D for d_theta /d_t
	PC_K = 0.2;					//    "    "         "       K  "  theta
	YC_D = 0.5	;				// Yaw Control
	YC_K = 0.3;
	PC_D_FINAL = 0.9;			// Pitch control coefficients for final section.
	PC_K_FINAL = 0.6; 
	D_FINAL  = 0.8;				// Vertical velocity control coefficients
	K_FINAL = 0.8;
	ENG_THRST_2 = 0 ;		// Thrust rating of Stage 2 engine. (N)  
	i_last = 0;					// value of incliniation in previous iteration.
	Planet_Name = new char[6];
	strcpy( Planet_Name, "Earth");
	R_EARTH = 6371000;			// radius of the earth (m)
	U_EARTH = 3.986e5 * 1000 * 1000 * 1000; //( m^3/sec^2)
	Ob_EARTH = 23.45 * RAD ;	// Obliquity of the Earth (rad)
	APO_ALT = 220 ;				// Altitude of apoapsis
//	APO_ALT = 1705;

	//  inclination correction factor constant. (3580)
	ICF_K = 62.008 * INCLINATION*INCLINATION 
			- 6848.7*INCLINATION + 189807;	

	DisplayIncSP = false;		// toggle for displaying the inclination setpoint.
//	PadHeading = 242.98;		// Heading of the Soyuz on the pad (deg)
//	PadHeading = 270.00;
								// Default heading is for inclination 51.6 deg.
	NewIncSP = false ;			// True if a user has entered a new inclination SP
	PostMECO = false ;			// switch for cleaning up after MECO

	BadHeading = false;			// true if soyuz is at an azimuth not suitable for the G&CS

//	PadHeading = 270.00;


}

double CVELTITAN::GetCPitch(double t)
{
	int i;
	if (t>met[N-1]) return cpitch[N-1];
	i=1;
	while (met[i]<t) i++;
	return cpitch[i-1]+(cpitch[i]-cpitch[i-1])/(met[i]-met[i-1])*(t-met[i-1]);
}

// rjcroy Lateral Guidance System ( called by Guidance_Routine() ).

void CVELTITAN::Lateral_Guidance(VESSEL *vessel, double SimT, double SimT_Last)
{
	// Lateral Guidance System 

		double hdg, diff, rate, amount, targetrate, TgtHdg;

		if (MET > 55.0) {
			targetrate = ((GetBank()*180/PI)-180)/-1.0;
			if (GetBank()<0) targetrate = -1.0*targetrate;

			if (MET > 75.0) {

				oapiGetHeading(GetHandle(),&hdg);
//				hdg = vst.arot.y * 180/PI;
				hdg *= 180/PI;
//				hdg += 180;

				GetRelativeVel (hEarth, Rel_Vel);
				GetEquPos( Longitude, Latitude, Radius );
				Velocity = sqrt((Rel_Vel.x*Rel_Vel.x) + (Rel_Vel.y*Rel_Vel.y) + (Rel_Vel.z*Rel_Vel.z));

				oapiGetRelativeVel(hVessel, hEarth, &Rel_Vel);
				oapiGetRelativePos(hVessel, hEarth, &Rel_Pos);
				
				// Change ecliptic rel. vectors to equatorial vectors.
 				temp.x = Rel_Vel.x;
				temp.y = Rel_Vel.z;
				temp.z = Rel_Vel.y;
				veccpy( Rel_Vel, temp);
				temp.x = Rel_Pos.x;
				temp.y = Rel_Pos.z;
				temp.z = Rel_Pos.y;
				veccpy( Rel_Pos, temp);
				temp.x = Rel_Pos.x ;
				temp.y = cos(-Ob_EARTH)* Rel_Pos.y + sin(-Ob_EARTH)*Rel_Pos.z;
				temp.z =  -sin(-Ob_EARTH)* Rel_Pos.y + cos(-Ob_EARTH)*Rel_Pos.z;
				veccpy( Rel_Pos, temp);
				temp.x = Rel_Vel.x ;
				temp.y = cos(-Ob_EARTH)* Rel_Vel.y + sin(-Ob_EARTH)*Rel_Vel.z;
				temp.z =  -sin(-Ob_EARTH)* Rel_Vel.y + cos(-Ob_EARTH)*Rel_Vel.z;
				veccpy( Rel_Vel, temp);

				h_vector = crossp(Rel_Vel, Rel_Pos);
				h_abs = sqrt((h_vector.x*h_vector.x) + (h_vector.y*h_vector.y) + (h_vector.z*h_vector.z));
				i = 180 - acos(h_vector.z / h_abs) * DEG;
				i_dot = (i - i_last) / (MET - SimT_Last) ;
				i_last = i;
				ICF = Velocity / ICF_K;

				// Calculated error in inclination as output signal
				i_err = (i - INCLINATION) * (150 * ICF) + 250 * ICF * i_dot;
				if (i_err > ICF )
				{	
					i_err = ICF;
				}
				else if ( i_err < -ICF) 
				{
					i_err = -ICF;
				}
							
				double Phrad;
					//= (PadHeading-180)*PI/180;

				if ((Rel_Vel.z < 110.0) || (bgoingsouth)) {
					Phrad = PI/2 + (PI/2 - asin(cos(INCLINATION*PI/180)/cos(Latitude)));
					if ((PadHeading-180) > 180)
						Phrad = 3*PI/2 + (3*PI/2 - asin(cos(INCLINATION*PI/180)/cos(Latitude)));
					TgtHdg = Phrad*DEG;
					TgtHdg -= i_err;
					bgoingsouth = true;
				}
				else {
					Phrad =  asin(cos(INCLINATION*PI/180)/cos(Latitude));
					TgtHdg = Phrad*DEG;
					TgtHdg += i_err;
				}

				if ((TgtHdg > 90.0) || (PadHeading-180 > 90.0))
					bgoingsouth = true;

				if ((Latitude*DEG >= i*0.98) && (fabs(i-INCLINATION)<0.02))
					bgoingsouth = true;

//				sprintf(oapiDebugString(),"Latitude %.2f  inc %.2f TGTINC %.2f going %1d",Latitude*DEG, i, INCLINATION, bgoingsouth);
//				sprintf(oapiDebugString(),"TgtHdg %.2f  inc %.2f TGTINC %.2f Phrad %.2f PadHdg %.2f tmpy %.2f",TgtHdg, i, INCLINATION, Phrad*DEG,PadHeading-180,Rel_Vel.z);

				diff = hdg - (TgtHdg);
//				diff = hdg - (PadHeading - 180);
				if( diff > 180.0 )
					diff -= 360.0;
				else if( diff < -180.0 )
					diff += 360.0;
				targetrate = diff/+1.0;
			}

			if( targetrate > 8.0 )
				targetrate = 8.0;
			else if( targetrate < -8.0 )
				targetrate = -8.0;


			rate = vst.vrot.y * 180/PI;
			amount = fabs (targetrate/8.0);
			if (MET > 0) amount = amount / (MET/10);

//		sprintf(oapiDebugString(),"Bank %.1f Heading %.1f diff %.1f Targetrate %.1f",GetBank()*180/PI,hdg,diff, targetrate);

			if( fabs( rate-targetrate ) > 0.05 ) {
				if( rate < targetrate ) {
//					IncThrusterGroupLevel( THGROUP_ATT_YAWRIGHT, 0.05 );
//					SetThrusterGroupLevel( THGROUP_ATT_YAWLEFT, 0.0 );
					yawinput = -1*amount;
				}
				else {
//					IncThrusterGroupLevel( THGROUP_ATT_YAWLEFT, 0.05);
//					SetThrusterGroupLevel( THGROUP_ATT_YAWRIGHT, 0.0 );
					yawinput = amount;
				}
			}
			else {
//				SetThrusterGroupLevel( THGROUP_ATT_YAWLEFT, 0.0 );
//				SetThrusterGroupLevel( THGROUP_ATT_YAWRIGHT, 0.0 );
				yawinput = 0.0;
			}
		}
}

// rjcroy Guidance & Control System
void CVELTITAN::Guidance_Routine(VESSEL *vessel, double SimT, double SimT_Last)
{

	hVessel = GetHandle();
	hEarth = oapiGetGbodyByName ( Planet_Name );


	if (T_minus_10)
	{
		// Record launch time
		if (TZero == 0){
			TZero = SimT + 11;
		}

		if (MET > -5 && MET < -3) {
//			oapiSetEngineLevel(hVessel, ENGINE_MAIN, 0.3);
//			LaunchClamps();
		}
		else if (MET > -3 && MET < 0) {
//			oapiSetEngineLevel(hVessel, ENGINE_MAIN, (0.2*MET + 1));
//			LaunchClamps();
			INCLINATION = 180 - acos(sin(PadHeading*PI/180)*cos(Latitude))*DEG;
		}
		else if (MET > 0)
		{
			T_minus_10 = false;
			First_Guidance_Pass = true;
			oapiSetEngineLevel(hVessel, ENGINE_MAIN, 1);

			GetStatusEx(&vst);
	
//			initialheading = vs.surf_hdg * 180/PI;
//			initialheading = 245.0;
			initialgamma = vst.arot.z * 180/PI;
			launchrotate =	PadHeading - initialheading;

			if( launchrotate < -180.0 )
				launchrotate += 360.0;
			else if( launchrotate > 180.0 )
				launchrotate -= 360.0;

			screenheight = 140.0;
			if (SSTOcase) screenheight = 95.0;
			if (UprS > 1) screenheight = 165.0;

		}
		if (oapiGetFocusObject() == GetHandle())
			sprintf(oapiDebugString(),"T - %.1f", fabs(MET));
	}		

	else if (First_Guidance_Pass) 
	{
		
		// Start the main engines
		oapiSetEngineLevel(hVessel, ENGINE_MAIN, 1);

		// Unset the first pass flag
		First_Guidance_Pass = false;
	}

	else if (!Alt_Flag) 
	{

		if (initialgamma == 0.0)
			initialgamma = vst.arot.z * 180/PI;

		// Get correct pitch from table
		Pitch_SP=GetCPitch(MET);
		MJD_Time = oapiGetSimMJD();
		GetElements( elements, MJD_Time );
		Alt_Apoapsis = (elements.a * ( 1 + elements.e )/1000) - 6371; 
		if ((Alt_Apoapsis > screenheight) || (Flydown)) {
			Flydown = true;
			GetHorizonAirspeedVector(horiz);

			Pitch_SP = Pitch_SP/4.0;			// Flydown -- we've met our altitude goal
			double Pitch_zero = -0.02*horiz.y;
			if ((TitanC) || (SSTOcase) || (Titan2) || (Titan1)) Pitch_SP = 0.0;
			if (SSTOcase)
				Pitch_SP += Pitch_zero-0.7*GetAOA()*DEG;
			else 
				Pitch_SP = Pitch_zero-0.85*GetAOA()*DEG;			
		}

		if (Pitch_SP < 0.0)
			Pitch_SP = 0.0;

//		sprintf(oapiDebugString(),"Alt_Apoapsis %.1f Pitch_SP %.1f CPitch %.1f",Alt_Apoapsis, Pitch_SP, GetCPitch(MET));

		// Control to pitch setpoint
		Pitch=GetPitch(  );
		Pitch = Pitch * DEG;
		Pitch_Error = Pitch - Pitch_SP;

		GetAngularVel( Angular_Velocity ); 
		Pitch_Error_Dot = Angular_Velocity.x * DEG; // get pitch change rate in degrees/s

		Pitch_Cntrl_Sig =  - PC_D * Pitch_Error_Dot + PC_K * Pitch_Error;
//		if ((MET > 10.0) &&(GetBank() < PI/2) && (GetBank() > -PI/2))
//			Pitch_Cntrl_Sig *= -1.0;

		if (Pitch_Cntrl_Sig > 1 )
		{
//			SetAttitudeRotLevel( 0, 1); // 0 is y-axis
			pitchinput =-1.0;
		}
		else if (Pitch_Cntrl_Sig < -1) 
		{
//			SetAttitudeRotLevel( 0, -1);
			pitchinput = +1.0;
		}
		else {
//			SetAttitudeRotLevel( 0 , Pitch_Cntrl_Sig);
			pitchinput = -Pitch_Cntrl_Sig;
		}
		

		if (MET > 190) {
			double diff = -1.0*GetBank()*180/PI;
			double yawangle = DEG*GetSlipAngle();
			yawinput = yawangle / 15.0;
		}
		else {
		// Heading control
		Lateral_Guidance(this, SimT, SimT_Last);
		}

		// Roll control
		// Adapted from Hendo's Ariane V


		if( MET > 10.0) {
				
		vst.version = 2;
		vst.flag = 0;
		vessel->GetStatusEx(&vst);


		double target = initialgamma - launchrotate;
		if( target > 180.0 )
			target -= 360.0;
		else if( target < -180.0 )
			target += 360.0;
					
		double diff = vst.arot.z*180/PI - target;
		if( diff > 180.0 )
			diff -= 360.0;
		else if( diff < -180.0 )
			diff += 360.0;

		if (MET > 50.0) diff = 0.0;
//		if ((MET > 150.0) && (fabs(fabs(GetBank()) - PI) < .2)) diff = GetBank()*180/PI + target;
		if (MET > 100) diff = GetBank()*180/PI + target;
//		if (MET > 250) diff = 0.0;

//		sprintf(oapiDebugString(),"initialhdg %.1f initialgamma %.1f launchrotate %.1f target %.1f diff %.1f TGT_ %.1f",initialheading,initialgamma, launchrotate, target,diff, PadHeading);

		double targetrate = diff/-1.0;
		if( targetrate > 8.0 )
			targetrate = 8.0;
		else if( targetrate < -8.0 )
			targetrate = -8.0;


		double rate = vst.vrot.z * 180/PI;
		double amount = fabs (targetrate/4.0);
		if (MET > 0) amount = amount / (MET/10);

		if( fabs( rate-targetrate ) > 0.05 ) {
			if( rate < targetrate ) {
//				IncThrusterGroupLevel( THGROUP_ATT_BANKRIGHT, 0.05 );
//				SetThrusterGroupLevel( THGROUP_ATT_BANKLEFT, 0.0 );
				rollinput = -1*amount;
			}
			else {
//				IncThrusterGroupLevel( THGROUP_ATT_BANKLEFT, 0.05);
//				SetThrusterGroupLevel( THGROUP_ATT_BANKRIGHT, 0.0 );
				rollinput = amount;
			}
		}
		else {
//			SetThrusterGroupLevel( THGROUP_ATT_BANKLEFT, 0.0 );
//			SetThrusterGroupLevel( THGROUP_ATT_BANKRIGHT, 0.0 );
			rollinput = 0.0;
		}
		
//		if ((MET > 60.0) && (diff < 0.5))

//		if (MET > 60.0)
//			Lateral_Guidance(this, SimT, SimT_Last);
//		else
//			yawinput = 0.0;

//		GetAngularVel(AngularVel);

//		yawinput=(-20*AngularVel.y);		

		// END Hendo's roll control


		}

		Altitude=GetAltitude();
		a = ((APO_ALT * 1000 + R_EARTH) + (Altitude + R_EARTH)) / 2;
		e = ((APO_ALT * 1000 + R_EARTH) / a ) - 1;
		h = sqrt(U_EARTH * a * ( 1 - (e * e)));
		V_periapsis = h / (Altitude + R_EARTH);

		
		// Get current velocity
		GetRelativeVel (hEarth, Rel_Vel);
		Velocity = sqrt((Rel_Vel.x*Rel_Vel.x) + (Rel_Vel.y*Rel_Vel.y) + (Rel_Vel.z*Rel_Vel.z));

		if ((autoLEO) && (MET > 180) && (Velocity > V_periapsis)) 
		{
			MECO_Cond = true;
			TimeMECO = SimT;
			oapiSetEngineLevel(hVessel, ENGINE_MAIN, 0);
			Guidance(false);
		}

		
	}

	else if (MECO_Cond && PostMECO) 
	{
		if ((SimT - TimeMECO) < 10) {
			sprintf(oapiDebugString(), "Guidance OFF.  Inclination (Equatorial): %.3f", i); 
		}	
		else {
			guidance = false;
			sprintf(oapiDebugString(), "");
		}
	}
}
// END Guidance_Routine()
/*
// Guidance user interface
void CVELTITAN::GetInclinationSP()
{
	// Gets a new Inclination setting from the pilot
//	bool IncInput (void *id, char *str, void *data);
	
//	oapiOpenInputBox ("Inclination Setpoint, e.g. 5163 = 51.63 deg (47 to 56)", IncInput, 0, 25);
}
*/
/*
bool CVELTITAN::IncInput (void *id, char *str, void *data)
{
	sscanf(str, "%4lf", &INCLINATION);
	if (INCLINATION > 4700 && INCLINATION < 5600 ) {
		INCLINATION = INCLINATION / 100;
		NewIncSP = true;
	}

	return 1 ;
}
*/

void CVELTITAN::GetInclinationSP()
{
	// Gets a new Inclination setting from the pilot
	bool AzInput (void *id, char *str, void *data);
	
//	oapiOpenInputBox ("Inclination Setpoint, e.g. 5163 = 51.63 deg (47 to 56)", IncInput, 0, 25);
	oapiOpenInputBox ("Enter launch azimuth, 0 to 90 (east) to 360", AzInput, 0, 20, (void*)this);

//	GetEquPos( Longitude, Latitude, Radius );
//	sprintf(oapiDebugString(), "Target inclination (Equatorial): %.3f", acos(sin(PadHeading*PI/180)*cos(Latitude))*DEG); 
}


bool AzInput (void *id, char *str, void *data)
{
//	sscanf(str, "%4lf", &PadHeading);
//	PadHeading = PadHeading / 100.0;
//	PadHeading += 180.0;
/*	sscanf(str, "%4lf", &INCLINATION);
	if (INCLINATION > 4700 && INCLINATION < 5600 ) {
		INCLINATION = INCLINATION / 100;
		NewIncSP = true;
	}
*/
	return ((CVELTITAN*)data)->IncInput (str);
}

bool CVELTITAN::IncInput (char *rstr)
{
	sscanf(rstr, "%5lf", &PadHeading);
//	PadHeading = PadHeading / 100.0;
	PadHeading += 180.0;
	if (PadHeading > 450.0) PadHeading -= 360.0;
	d_heading = PadHeading;
	displaytime = 0.01;
	return true;
}

// END rjcroy Guidance System
CVELTITAN::CVELTITAN (OBJHANDLE hObj, int fmodel)
: COMMON_VESSEL_EX (hObj)
{

	bManualSeparate = false;
	bThrustTerm = false;
	bAborting = false;
	stage_sep=0;
	stage = 3;
	status = 3;
	invoked = true;
	ignition= false;
	time=0.0;
	autopilot = false;
	engineignited = false;
	bgoingsouth = false;
	bjustseparated = true;
	SecondStageRCS = false;
	last_simt=0.0;
	ignition_time=0.0;
	density = 0.1;
	refcount = 0;
	SecndS = 1;
	autoLEO = false;
	padbias = 0.0;
	LaunchMJD = 0.0;
	simt = 0.0;
	oldsimt = -0.01;
	displaytime = 0.01;
	deployed = 0;
	shiftfocus = 0;
	UprS = 0;
	Uppermass = 0.0;
	BoostersSet = false;
	ph_SRMs0 = NULL;
	ph_SRMs1 = NULL;
	ph_first = NULL;
	ph_second = NULL;
	ph_upper = NULL;
	PadHeading = 270.0;
	mass_fairing = 0.0;
	LastDeployed = 0.0;
	Firstsweep = true;
	atchmass = 0.0;
	LEStime = 0.0;
	Flydown = false;
	ClusterDecreased = false;

	mode = 0;
	hV = NULL;
	stage_sep = 0;
	bManualSeparate = false;
	firsttime = true;
	fairing = 0;
	DrawAGemini = 0;
	GeminiZ = 0;
	ofs_shroud1 = OFS_SHROUD1;
	ofs_shroud2 = OFS_SHROUD2;
	ofs_shroud3 = OFS_SHROUD3;
	numSRMs1 = 0;
	numSRMs0 = 0;
	OFFSIDEL = _V(0,0,0);
	OFFSIDER = _V(0,0,0);

	Stg0Longer = 0.0;
	Stg1Longer = 0.0;
	Stg1Bias = 0.0;
	Stg2Bias = 0.0;
	Stg2EngBias = 0.0;
	Stg2Longer = 0.0;
	OutBias = 0.12;

	Titan2 = false;
	Titan3 = false;
	Titan34 = false;
	Titan4 = false;
	Titan3M = false;
	TitanC = false;
	Cluster = false;
	SRM_Type = UA1205;
	EnginesVert = false;
	SRMBias = 0;
	SRMExhBias = 0;
	bswitchfocus = false;
	bsolidsfired = false;
	deploy_no_more = false;
	vectorme = true;
	invoked = true;
	stg1invoked = false;
	CAMERA_OFS = _V(0,0,0);
	horiz = _V(0,0,0);
	mass_first = MASS_FIRST23;
	mass_first_fuel = MASS_FIRST23_FUEL;
	mass_first_empty = MASS_FIRST23_EMPTY;
	mass_second = MASS_SECOND23;
	mass_second_fuel = MASS_SECOND23_FUEL;
	mass_second_empty = MASS_SECOND23_EMPTY;

	attach = _V(0,0,-1.0);
	attachdir = _V(0,0,1);
	attachrot = _V(0, 1,0);
	//	  		       			     child   pos           dir           rot      handle
	attachpos = CreateAttachment (false, attach,   attachdir, attachrot, "X");



	MyID=ConnectToOrbiterSoundDLL3(GetHandle());
	RequestLoadVesselWave3(MyID,PLAYABORT,"Sound\\X20\\fasep.wav",BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(MyID,PLAYBANG,"Sound\\X20\\stg2sep.wav",BOTHVIEW_FADED_FAR);

	char classname[30];
	int i;
	strncpy( classname, GetClassName(), 29 );
	classname[29]=0;
	for( i = 0 ; i < strlen( classname ) ; i++ ) {
		classname[i] = toupper(classname[i]);
	}
	if( strstr( classname, "CVELTITANIIIM" ) ) {
		Titan34 = true;
		SRM_Type = UA1207;
	}
	else if( strstr( classname, "CVELTITAN2STG1" ) ) {
		Titan2 = true;
		status = 2;
		stage = 2;
		UprS = 0;
		SecndS = 0;
		invoked = true;
		stg1invoked = true;
	}
	else if( strstr( classname, "CVELTITANII" ) ) {
		Titan2 = true;
	}
	else if( strstr( classname, "CVELTITAN34" ) ) {
		Titan34 = true;
	}
	else if( strstr( classname, "CVELTITANIVB" ) ) {
		Titan4 = true;
		SRM_Type = SRMU;
	}
	else if( strstr( classname, "CVELTITANIV" ) ) {
		Titan4 = true;
		SRM_Type = UA1207;
	}
	else if( strstr( classname, "CVELTITANCLUSTER" ) ) {
		Titan1 = true;
		Cluster = true;
		status = 0;
		stage = 0;
		SecndS = 0;
		UprS = 0;
		hCluster = oapiLoadMeshGlobal ("CVELTitan/titanclust");
		hClusterInt = oapiLoadMeshGlobal ("CVELTitan/titanclint");
	}
	else if( strstr( classname, "CVELTITANC" ) ) {
		TitanC = true;
	}
	else if( strstr( classname, "CVELTITANI" ) ) {
		Titan1 = true;
	}
	else
		Titan3 = true;

	if (Titan2) {
		hFirstStage = oapiLoadMeshGlobal ("CVELTitan/titan21");
		hInterStage = oapiLoadMeshGlobal ("CVELTitan/titan2i");
		hSecondStage = oapiLoadMeshGlobal ("CVELTitan/Titan22");
	}
	else if (Titan1) {
		hFirstStage = oapiLoadMeshGlobal ("CVELTitan/titan21");
		hInterStage = oapiLoadMeshGlobal ("CVELTitan/Int10_8");
		hSecondStage = oapiLoadMeshGlobal ("CVELTitan/TitanI_Stage2");
		Stg2Longer = 1.00;
		Stg2EngBias = -0.80;
		Stg2Bias = -1.15;
		Stg1Bias = 0.25;
	}
	else if (Titan4) {
		hFirstStage = oapiLoadMeshGlobal ("CVELTitan/Titan45_Stage1");
		hInterStage = oapiLoadMeshGlobal ("CVELTitan/titan2i");
		hSecondStage = oapiLoadMeshGlobal ("CVELTitan/Titan45_Stage2");
		Stg1Longer = -4.24;
		Stg1Bias = 1.98;  //-2.08;
		Stg2Bias = 0.0;
		Stg2EngBias = -0.65;
		EnginesVert = true;
	}
	else if (TitanC) {
		hFirstStage = oapiLoadMeshGlobal ("CVELTitan/titan21");
		hInterStage = oapiLoadMeshGlobal ("CVELTitan/int10_13");
		hSecondStage = oapiLoadMeshGlobal ("CVELTitan/TitanC_Stage2");
		Stg2Longer = -8.15;
		Stg2Bias = 8.15;
		Stg2EngBias = -0.65;
	}
	else {
		hFirstStage = oapiLoadMeshGlobal ("CVELTitan/Titan34_Stage1");
		hInterStage = oapiLoadMeshGlobal ("CVELTitan/titan2i");
		hSecondStage = oapiLoadMeshGlobal ("CVELTitan/Titan34_Stage2");
		Stg1Longer = -1.81;
		Stg1Bias = -0.45; //-2.05;
		Stg2Bias = 0.0;
		Stg2EngBias = -0.65;
		EnginesVert = true;
	}
	hTranstage = oapiLoadMeshGlobal ("transtage");
	hInterStage2 = oapiLoadMeshGlobal ("TenFtInterstage");
	hInterStage2b = oapiLoadMeshGlobal ("TentoFiveFtInterstage");
	hInterStage2c = oapiLoadMeshGlobal ("CVELTitan/int13_10");
	hCentaur = oapiLoadMeshGlobal ("CentaurD");
	hAgena = oapiLoadMeshGlobal ("CVELTitan/CVELAgena");
	hChariot = oapiLoadMeshGlobal ("CVELTitan/Chariot");
	hSRM0 = oapiLoadMeshGlobal ("CVELTitan/SRM0");
	hSRM1 = oapiLoadMeshGlobal ("CVELTitan/SRM1");
	hSRM2 = oapiLoadMeshGlobal ("CVELTitan/SRM2");
	hSRM3 = oapiLoadMeshGlobal ("CVELTitan/SRM3");
	hSRM4 = oapiLoadMeshGlobal ("CVELTitan/SRM4");
	hSRM5 = oapiLoadMeshGlobal ("CVELTitan/SRM5");
	hSRM6 = oapiLoadMeshGlobal ("CVELTitan/SRM6");
	hSRM7 = oapiLoadMeshGlobal ("CVELTitan/SRM7");
	hSRM8 = oapiLoadMeshGlobal ("CVELTitan/SRM8");
	hSRM9 = oapiLoadMeshGlobal ("CVELTitan/SRM9");

	hGemEquip = NULL;
	hGemAdapt = NULL;
	hGemCapsule = NULL;
	hGemHeatshield = NULL;
	hGemCdrSeat = NULL;
	hGemPltSeat = NULL;
	hGemDoorL = NULL;
	hGemDoorR = NULL;
	hGemNose = NULL;


	isp_srm_sl			= ISP_UA1205_SL; 
	isp_srm_vac			= ISP_UA1205_VAC;
	isp_main_sl			= ISP_FIRST_SL;
	isp_main_vac		= ISP_FIRST_VAC;
	isp_second_sl		= ISP_SECOND_SL;
	isp_second_vac		= ISP_SECOND_VAC;
	thrust_main_vac		= THRUST_FIRST_VAC;
	thrust_second_vac	= THRUST_SECOND_VAC;

	if ((Titan2) || (Titan3)) {
		mass_first = MASS_FIRST23;
		mass_first_fuel = MASS_FIRST23_FUEL;
		mass_first_empty = MASS_FIRST23_EMPTY;
		mass_second = MASS_SECOND23;
		mass_second_fuel = MASS_SECOND23_FUEL;
		mass_second_empty = MASS_SECOND23_EMPTY;
	}
	else if (Titan1) {
		mass_first = MASS_FIRST1;
		mass_first_fuel = MASS_FIRST1_FUEL;
		mass_first_empty = MASS_FIRST1_EMPTY;
		mass_second = MASS_SECOND1;
		mass_second_fuel = MASS_SECOND1_FUEL;
		mass_second_empty = MASS_SECOND1_EMPTY;
		thrust_main_vac		= THRUST_FIRST1_VAC;
		thrust_second_vac	= THRUST_SECOND1_VAC;
		isp_main_sl			= ISP_FIRST1_SL;
		isp_main_vac		= ISP_FIRST1_VAC;
		isp_second_sl		= ISP_SECOND1_SL;
		isp_second_vac		= ISP_SECOND1_VAC;
	}
	else if (Titan34) {
		mass_first = MASS_FIRST34;
		mass_first_fuel = MASS_FIRST34_FUEL;
		mass_first_empty = MASS_FIRST34_EMPTY;
		mass_second = MASS_SECOND34;
		mass_second_fuel = MASS_SECOND34_FUEL;
		mass_second_empty = MASS_SECOND34_EMPTY;
		isp_main_sl			= ISP_FIRST34_SL;
		isp_main_vac		= ISP_FIRST34_VAC;
		isp_second_sl		= ISP_SECOND34_SL;
		isp_second_vac		= ISP_SECOND34_VAC;
		thrust_main_vac		= THRUST_FIRST34_VAC;
		thrust_second_vac	= THRUST_SECOND34_VAC;
	}
	else if (Titan4) {
		mass_first = MASS_FIRST45;
		mass_first_fuel = MASS_FIRST45_FUEL;
		mass_first_empty = MASS_FIRST45_EMPTY;
		mass_second = MASS_SECOND45;
		mass_second_fuel = MASS_SECOND45_FUEL;
		mass_second_empty = MASS_SECOND45_EMPTY;
		isp_main_sl			= ISP_FIRST34_SL;
		isp_main_vac		= ISP_FIRST34_VAC;
		isp_second_sl		= ISP_SECOND34_SL;
		isp_second_vac		= ISP_SECOND34_VAC;
		thrust_main_vac		= THRUST_FIRST34_VAC;
		thrust_second_vac	= THRUST_SECOND34_VAC;
	}
	else if (TitanC) {
		mass_first = MASS_FIRST23;
		mass_first_fuel = MASS_FIRST23_FUEL;
		mass_first_empty = MASS_FIRST23_EMPTY;
		mass_second = MASS_SECONDC;
		mass_second_fuel = MASS_SECONDC_FUEL;
		mass_second_empty = MASS_SECONDC_EMPTY;
		isp_second_sl		= ISP_C_SL;
		isp_second_vac		= ISP_C_VAC;
		thrust_second_vac	= THRUST_C_VAC;
		thrust_main_vac		= THRUST_FIRST34_VAC;
		isp_main_sl			= ISP_FIRST34_SL;
		isp_main_vac		= ISP_FIRST34_VAC;
	}


	CheckCentralMessage();   

 
	key = (char*)VirtualAlloc(NULL,256,MEM_COMMIT,PAGE_READWRITE);
	message = (char*)VirtualAlloc(NULL,256,MEM_COMMIT,PAGE_READWRITE);
}


CVELTITAN::~CVELTITAN()
{

//	delete []key;
//	delete []message;
	VirtualFree(key,256,MEM_DECOMMIT);
	VirtualFree(message,256,MEM_DECOMMIT);
}


void CVELTITAN::SetBoosters()
{
	BoostersSet = true;

	stage0_ofs_left			= STAGE0_OFS_LEFT;
	stage0_ofs_right		= STAGE0_OFS_RIGHT;
	stage0_ofs_left_flame	= STAGE0_OFS_LEFT_FLAME;
	stage0_ofs_right_flame	= STAGE0_OFS_RIGHT_FLAME;
	isp_srm_sl				= ISP_UA1205_SL; 
	isp_srm_vac				= ISP_UA1205_VAC;

	if (Cluster) {
		srm_prop_mass		= MASS_FIRST1_FUEL*2;
		mass_srms			= MASS_FIRST1_EMPTY*2;
		stage0_ofs_left = _V(0,0,STAGE1_OFS.z-22.75);
		stage0_ofs_left_flame = _V(0,0,STAGE1_OFS.z-22.75-11);
	}
	else if (SRM_Type == UA1207)  {
		srm_prop_mass		= MASS_UA1207_FUEL;
	 	mass_srms			= MASS_UA1207_EMPTY;	
		thrust_srm_vac		= THRUST_UA1207_VAC;
		hsrm1 = oapiLoadMeshGlobal ("CVELTitan/UA1207W");
		hsrm2 = oapiLoadMeshGlobal ("CVELTitan/UA1207E");
		stage0_ofs_left.z			+= 1.5;
		stage0_ofs_right.z			+= 1.5;
		stage0_ofs_left_flame.z		-= 4.0;
		stage0_ofs_right_flame.z	-= 4.0;
	}
	else if (SRM_Type == UA1208)  {
		srm_prop_mass		= MASS_UA1208_FUEL;
	 	mass_srms			= MASS_UA1208_EMPTY;	
		thrust_srm_vac		= THRUST_UA1208_VAC;
		hsrm1 = oapiLoadMeshGlobal ("CVELTitan/UA1208W");
		hsrm2 = oapiLoadMeshGlobal ("CVELTitan/UA1208E");
		stage0_ofs_left.z			+= 1.5;
		stage0_ofs_right.z			+= 1.5;
		stage0_ofs_left_flame.z		-= 4.0;
		stage0_ofs_right_flame.z	-= 4.0;
	}
	else if (SRM_Type == SRMU) {
		srm_prop_mass		= MASS_SRMU_FUEL;
	 	mass_srms			= MASS_SRMU_EMPTY;	
		thrust_srm_vac		= THRUST_SRMU_VAC;
		isp_srm_sl			= ISP_SRMU_SL; 
		isp_srm_vac			= ISP_SRMU_VAC;
		hsrm1 = oapiLoadMeshGlobal ("CVELTitan/SRMUW");
		hsrm2 = oapiLoadMeshGlobal ("CVELTitan/SRMUE");
		stage0_ofs_left.z			+= 1.5;
		stage0_ofs_right.z			+= 1.5;
		stage0_ofs_left_flame.z		-= 4.0;
		stage0_ofs_right_flame.z	-= 4.0;
		OutBias = 0.0;
	}
	else if (SRM_Type == RSRM) {
		srm_prop_mass		= MASS_RSRM_FUEL;
	 	mass_srms			= MASS_RSRM_EMPTY;	
		thrust_srm_vac		= THRUST_RSRM_VAC;
		isp_srm_sl			= ISP_RSRM_SL; 
		isp_srm_vac			= ISP_RSRM_VAC;
		hsrm1 = oapiLoadMeshGlobal ("CVELTitan/RSRMW");
		hsrm2 = oapiLoadMeshGlobal ("CVELTitan/RSRME");
		stage0_ofs_left			= RSRM_OFS_LEFT;
		stage0_ofs_right		= RSRM_OFS_RIGHT;
		stage0_ofs_left_flame	= RSRM_OFS_LEFT_FLAME;
		stage0_ofs_right_flame	= RSRM_OFS_RIGHT_FLAME;
		Stg0Longer = -8.5;
		OutBias = 0.0;
	}
	else if (SRM_Type == UA1205A) {
		srm_prop_mass		= MASS_UA1205_FUEL;
		mass_srms			= MASS_UA1205_EMPTY+1000.0;
		thrust_srm_vac		= THRUST_UA1205_VAC;
		hsrm1 = oapiLoadMeshGlobal ("CVELTitan/UA1205AW");
		hsrm2 = oapiLoadMeshGlobal ("CVELTitan/UA1205AE");
	}
	else if (SRM_Type == UA1003) {
		srm_prop_mass		= MASS_UA1003_FUEL;
		mass_srms			= MASS_UA1003_EMPTY;
		thrust_srm_vac		= THRUST_UA1003_VAC;
		isp_srm_sl			= ISP_UA1003_SL; 
		isp_srm_vac			= ISP_UA1003_VAC;
		stage0_ofs_left.x			+= 0.25;
		stage0_ofs_right.x			+= -0.25;
		stage0_ofs_left_flame.x		+= 0.25;
		stage0_ofs_right_flame.x	+= -0.25;
		stage0_ofs_left.z			+= 2.0;
		stage0_ofs_right.z			+= 2.0;
		stage0_ofs_left_flame.z		-= 2.0;
		stage0_ofs_right_flame.z	-= 2.0;
		hsrm1 = oapiLoadMeshGlobal ("CVELTitan/UA1003W");
		hsrm2 = oapiLoadMeshGlobal ("CVELTitan/UA1003E");
	}
	else {
		srm_prop_mass		= MASS_UA1205_FUEL;
		mass_srms			= MASS_UA1205_EMPTY;
		thrust_srm_vac		= THRUST_UA1205_VAC;
		hsrm1 = oapiLoadMeshGlobal ("CVELTitan/UA1205W");
		hsrm2 = oapiLoadMeshGlobal ("CVELTitan/UA1205E");
	}
}

void CVELTITAN::VectorThrust()
{

	if (firsttime) {
		firsttime = false;
		return;
	}
	if ((status == 4) && (!vectorme)) return;

	if (oapiGetTimeAcceleration() > 10) return;

	if (oapiGetFocusObject() == GetHandle()) {
		tempP = GetManualControlLevel(THGROUP_ATT_PITCHDOWN, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - 
			GetManualControlLevel(THGROUP_ATT_PITCHUP, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE);

		tempY = GetManualControlLevel(THGROUP_ATT_YAWLEFT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - 
			GetManualControlLevel(THGROUP_ATT_YAWRIGHT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE);

		tempR = GetManualControlLevel(THGROUP_ATT_BANKLEFT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - 
			GetManualControlLevel(THGROUP_ATT_BANKRIGHT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE);

	} else {
		tempP = 0.0;
		tempY = 0.0;
		tempR = 0.0;
	}
//		if ((status == 1) || (status == 2)) tempR *= 2.0;

		GetAngularVel(AngularVel);

		tempP+=(3*AngularVel.x);			// Bias the thrust vector AGAINST the angular velocity a bit
		tempY+=(-3*AngularVel.y);			// This will damp angular velocity, preventing loss of control
		tempR+=(3*AngularVel.z);
	

		tempP+=pitchinput;
		tempY+=yawinput;
		tempR+=rollinput;
	
		if (bjustseparated) {
			tempP = 0.0;
			tempY = 0.0;
			tempR = 0.0;
			bjustseparated = false;
		}

//		if (status == -1) tempP+=-0.59;

		if ((status < 1) && (!Cluster)) {
			SetThrusterDir(th_srb[0],_V(0.005*tempY+OutBias,0.005*(tempP+3*tempR),1));
			SetThrusterDir(th_srb[1],_V(0.005*tempY-OutBias,0.005*(tempP-3*tempR),1));
		}
		if ((status < 1) && (Cluster)) {
			SetThrusterDir(th_srb[0],_V(0.005*tempY,0.005*(tempP+3*tempR),1));
			SetThrusterDir(th_srb[1],_V(0.005*tempY,0.005*(tempP+3*tempR),1));
			SetThrusterDir(th_srb[2],_V(0.005*tempY,0.005*(tempP-3*tempR),1));
			SetThrusterDir(th_srb[3],_V(0.005*tempY,0.005*(tempP-3*tempR),1));
		}
		if ((status == 1) || (status == 2)) {
			if (!EnginesVert) {
				SetThrusterDir(th_main[0],_V(0.004*tempY,0.004*(tempP-8*tempR),1));
				SetThrusterDir(th_main[1],_V(0.004*tempY,0.004*(tempP+8*tempR),1));
			}
			else {
				SetThrusterDir(th_main[0],_V(0.004*(tempY+8*tempR),0.004*(tempP),1));
				SetThrusterDir(th_main[1],_V(0.004*(tempY-8*tempR),0.004*(tempP),1));
			}
		}
		if ((status > 2) && (SecndS > 0)) {
			SetThrusterDir(th_second[0],_V(0.008*tempY+.0007,0.008*(tempP+0.45*tempR),1));
			SetThrusterDir(th_second[1],_V(0,-0.32*tempR,1));
		}	
		if ((status > 3) && (UprS > 0)) {
			if ((UprS == 3) || (UprS == 4)) 	{
				SetThrusterDir(th_upper[0],_V(0.008*tempY,0.008*tempP,1));
				SetAttitudeRotLevel( 2, -0.25*tempR); // 2 is z-axis
			}
			else {
				SetThrusterDir(th_upper[0],_V(0.008*(tempY-20*tempR),0.008*(tempP),1));
				SetThrusterDir(th_upper[1],_V(0.008*(tempY+20*tempR),0.008*(tempP),1));
			}
		}	

}

void CVELTITAN::StageSRMs ()
{

	VESSELSTATUS vs;
	VECTOR3 ofs, rofs, vel;
	char name1[256], name2[256], name3[256], name4[256], name5[256], name6[256], name7[256], name8[256], name9[256], name10[256];
	strcpy (name1, GetName());
	strcpy (name2, GetName());
	strcpy (name3, GetName());
	strcpy (name4, GetName());
	strcpy (name5, GetName());
	strcpy (name6, GetName());
	strcpy (name7, GetName());
	strcpy (name8, GetName());
	strcpy (name9, GetName());
	strcpy (name10, GetName());

	if (numSRMs0 > 0) {
		GetStatus (vs);
		vs.status=0;
		ofs = SRM4_OFS+OFFSIDEL;
		vel = SRM4_OFS/2.0;
		vel.z = -1.5;
		ofs.z += Stg0Longer;
		Local2Rel (ofs, vs.rpos);
		rofs = vs.rvel;
		GlobalRot (vel, rofs);
		vs.rvel.x = vs.rvel.x+rofs.x;
		vs.rvel.y = vs.rvel.y+rofs.y;
		vs.rvel.z = vs.rvel.z+rofs.z;
		vs.vrot.x = SRM4_OFS.y/16.0;
		vs.vrot.y = SRM4_OFS.x/-16.0;
		vs.vrot.z = 0.0;
		oapiCreateVessel(strcat (name5, " Small SRM 5"), "CVELTitan/SRM-W", vs);
		GetStatus (vs);
		vs.status=0;
		ofs = SRM5_OFS+OFFSIDER;
		vel = SRM5_OFS/2.0;
		vel.z = -1.5;
		ofs.z += Stg0Longer;
		Local2Rel (ofs, vs.rpos);
		rofs = vs.rvel;
		GlobalRot (vel, rofs);
		vs.rvel.x = vs.rvel.x+rofs.x;
		vs.rvel.y = vs.rvel.y+rofs.y;
		vs.rvel.z = vs.rvel.z+rofs.z;
		vs.vrot.x = SRM5_OFS.y/16.0;
		vs.vrot.y = SRM5_OFS.x/-16.0;
		vs.vrot.z = 0.0;
		oapiCreateVessel(strcat (name6, " Small SRM 6"), "CVELTitan/SRM-E", vs);
		GetStatus (vs);
		vs.status=0;
		ofs = SRM6_OFS+OFFSIDEL;
		vel = SRM6_OFS/2.0;
		vel.z = -1.5;
		ofs.z += Stg0Longer;
		Local2Rel (ofs, vs.rpos);
		rofs = vs.rvel;
		GlobalRot (vel, rofs);
		vs.rvel.x = vs.rvel.x+rofs.x;
		vs.rvel.y = vs.rvel.y+rofs.y;
		vs.rvel.z = vs.rvel.z+rofs.z;
		vs.vrot.x = SRM6_OFS.y/16.0;
		vs.vrot.y = SRM6_OFS.x/-16.0;
		vs.vrot.z = 0.0;
		oapiCreateVessel(strcat (name7, " Small SRM 7"), "CVELTitan/SRM-W", vs);
		GetStatus (vs);
		vs.status=0;
		ofs = SRM7_OFS+OFFSIDER;
		vel = SRM7_OFS/2.0;
		vel.z = -1.5;
		ofs.z += Stg0Longer;
		Local2Rel (ofs, vs.rpos);
		rofs = vs.rvel;
		GlobalRot (vel, rofs);
		vs.rvel.x = vs.rvel.x+rofs.x;
		vs.rvel.y = vs.rvel.y+rofs.y;
		vs.rvel.z = vs.rvel.z+rofs.z;
		vs.vrot.x = SRM7_OFS.y/16.0;
		vs.vrot.y = SRM7_OFS.x/-16.0;
		vs.vrot.z = 0.0;
		oapiCreateVessel(strcat (name8, " Small SRM 8"), "CVELTitan/SRM-E", vs);
		GetStatus (vs);
		vs.status=0;
		ofs = SRM8_OFS+OFFSIDEL;
		vel = SRM8_OFS/2.0;
		vel.z = -1.5;
		ofs.z += Stg0Longer;
		Local2Rel (ofs, vs.rpos);
		rofs = vs.rvel;
		GlobalRot (vel, rofs);
		vs.rvel.x = vs.rvel.x+rofs.x;
		vs.rvel.y = vs.rvel.y+rofs.y;
		vs.rvel.z = vs.rvel.z+rofs.z;
		vs.vrot.x = SRM8_OFS.y/16.0;
		vs.vrot.y = SRM8_OFS.x/-16.0;
		vs.vrot.z = 0.0;
		oapiCreateVessel(strcat (name9, " Small SRM 9"), "CVELTitan/SRM-W", vs);
		GetStatus (vs);
		vs.status=0;
		ofs = SRM9_OFS+OFFSIDER;
		vel = SRM9_OFS/2.0;
		vel.z = -1.5;
		ofs.z += Stg0Longer;
		Local2Rel (ofs, vs.rpos);
		rofs = vs.rvel;
		GlobalRot (vel, rofs);
		vs.rvel.x = vs.rvel.x+rofs.x;
		vs.rvel.y = vs.rvel.y+rofs.y;
		vs.rvel.z = vs.rvel.z+rofs.z;
		vs.vrot.x = SRM9_OFS.y/16.0;
		vs.vrot.y = SRM9_OFS.x/-16.0;
		vs.vrot.z = 0.0;
		oapiCreateVessel(strcat (name10, " Small SRM 10"), "CVELTitan/SRM-E", vs);

		numSRMs0 = 0;
		DelPropellantResource (ph_SRMs0);

	}
	else {
		GetStatus (vs);
		vs.status=0;
		ofs = SRM0_OFS;
		vel = SRM0_OFS/2.0;
		vel.z = -1.5;
		Local2Rel (ofs, vs.rpos);
		rofs = vs.rvel;
		GlobalRot (vel, rofs);
		vs.rvel.x = vs.rvel.x+rofs.x;
		vs.rvel.y = vs.rvel.y+rofs.y;
		vs.rvel.z = vs.rvel.z+rofs.z;
		vs.vrot.x = SRM0_OFS.y/16.0;
		vs.vrot.y = SRM0_OFS.x/-16.0;
		vs.vrot.z = 0.0;
		oapiCreateVessel(strcat (name1, " Small SRM 1"), "CVELTitan/SRM-N", vs);
		GetStatus (vs);
		vs.status=0;
		ofs = SRM1_OFS;
		vel = SRM1_OFS/2.0;
		vel.z = -1.5;
		Local2Rel (ofs, vs.rpos);
		rofs = vs.rvel;
		GlobalRot (vel, rofs);
		vs.rvel.x = vs.rvel.x+rofs.x;
		vs.rvel.y = vs.rvel.y+rofs.y;
		vs.rvel.z = vs.rvel.z+rofs.z;
		vs.vrot.x = SRM1_OFS.y/16.0;
		vs.vrot.y = SRM1_OFS.x/-16.0;
		vs.vrot.z = 0.0;
		oapiCreateVessel(strcat (name2, " Small SRM 2"), "CVELTitan/SRM-S", vs);

		if (numSRMs1 == 4) {
			GetStatus (vs);
			vs.status=0;
			ofs = SRM2_OFS;
			vel = SRM2_OFS/2.0;
			vel.z = -1.5;
			Local2Rel (ofs, vs.rpos);
			rofs = vs.rvel;
			GlobalRot (vel, rofs);
			vs.rvel.x = vs.rvel.x+rofs.x;
			vs.rvel.y = vs.rvel.y+rofs.y;
			vs.rvel.z = vs.rvel.z+rofs.z;
			vs.vrot.x = SRM2_OFS.y/16.0;
			vs.vrot.y = SRM2_OFS.x/-16.0;
			vs.vrot.z = 0.0;
			oapiCreateVessel(strcat (name3, " Small SRM 3"), "CVELTitan/SRM-S", vs);
 			GetStatus (vs);
			vs.status=0;
			ofs = SRM3_OFS;
			vel = SRM3_OFS/2.0;
			vel.z = -1.5;
			Local2Rel (ofs, vs.rpos);
			rofs = vs.rvel;
			GlobalRot (vel, rofs);
			vs.rvel.x = vs.rvel.x+rofs.x;
			vs.rvel.y = vs.rvel.y+rofs.y;
			vs.rvel.z = vs.rvel.z+rofs.z;
			vs.vrot.x = SRM3_OFS.y/16.0;
			vs.vrot.y = SRM3_OFS.x/-16.0;
			vs.vrot.z = 0.0;
			oapiCreateVessel(strcat (name4, " Small SRM 4"), "CVELTitan/SRM-N", vs);
		}
		
		numSRMs1 = 0;

		DelPropellantResource (ph_SRMs1);
	}

	DelThrusterGroup (thg_SRMs, THGROUP_USER, true);
	
	CalcWeights();

	MakeSRMs();

	ClearMeshesEx();
	PlaceComponents();
	Draw();

}


void CVELTITAN::MakeSRMs ()
{

/*  A little routine that ADDS the mass and thrust of small SRM's to whatever stage we call it with
*/

	if (numSRMs1 == 0) return;
	double Propmass, SRMmass, thrustvac, Ispvac, Ispsl;
	if (Titan2) {
		SRMmass = MASS_CASTOR_EMPTY*(numSRMs0+numSRMs1);
		Propmass = MASS_CASTOR_FUEL;
		thrustvac = THRUST_CASTOR_VAC;
		Ispvac = ISP_CASTOR_VAC;
		Ispsl = ISP_CASTOR_SL;
	}
	else {
		SRMmass = MASS_ALGOL_EMPTY*(numSRMs0+numSRMs1);
		Propmass = MASS_ALGOL_FUEL;
		thrustvac = THRUST_ALGOL_VAC;
		Ispvac = ISP_ALGOL_VAC;
		Ispsl = ISP_ALGOL_SL;
	}

	if (numSRMs0 > 0)
		if (!ph_SRMs0) ph_SRMs0 = CreatePropellantResource (Propmass*numSRMs0);

	if (!ph_SRMs1) ph_SRMs1 = CreatePropellantResource (Propmass*numSRMs1);

	
	PARTICLESTREAMSPEC srm_exhaust = {
		0, 1.0, 20, 150.0, 0.1, 0.3, 12, 2.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-6, 0.1
	};

	DelThrusterGroup (thg_SRMs, THGROUP_USER, true);

	if (numSRMs0 > 0) {
		th_SRMs[0] = CreateThruster (SRM4_OFS+OFFSIDEL+_V(0,0,Stg0Longer+Stg1Longer+Stg2Longer), _V(-SRM4_OFS.x/25,-SRM4_OFS.y/25,1), thrustvac, ph_SRMs0, Ispvac, Ispsl);
		th_SRMs[1] = CreateThruster (SRM5_OFS+OFFSIDER+_V(0,0,Stg0Longer+Stg1Longer+Stg2Longer), _V(-SRM5_OFS.x/25,-SRM5_OFS.y/25,1), thrustvac, ph_SRMs0, Ispvac, Ispsl);
		th_SRMs[2] = CreateThruster (SRM6_OFS+OFFSIDEL+_V(0,0,Stg0Longer+Stg1Longer+Stg2Longer), _V(-SRM6_OFS.x/25,-SRM6_OFS.y/25,1), thrustvac, ph_SRMs0, Ispvac, Ispsl);
		th_SRMs[3] = CreateThruster (SRM7_OFS+OFFSIDER+_V(0,0,Stg0Longer+Stg1Longer+Stg2Longer), _V(-SRM7_OFS.x/25,-SRM7_OFS.y/25,1), thrustvac, ph_SRMs0, Ispvac, Ispsl);
		th_SRMs[4] = CreateThruster (SRM8_OFS+OFFSIDEL+_V(0,0,Stg0Longer+Stg1Longer+Stg2Longer), _V(-SRM8_OFS.x/25,-SRM8_OFS.y/25,1), thrustvac, ph_SRMs0, Ispvac, Ispsl);
		th_SRMs[5] = CreateThruster (SRM9_OFS+OFFSIDER+_V(0,0,Stg0Longer+Stg1Longer+Stg2Longer), _V(-SRM9_OFS.x/25,-SRM9_OFS.y/25,1), thrustvac, ph_SRMs0, Ispvac, Ispsl);
		AddExhaustStream (th_SRMs[0], SRM4_OFS+OFFSIDEL+_V(0,0,Stg0Longer+Stg1Longer+Stg2Longer-5.9), &srm_exhaust);
		AddExhaustStream (th_SRMs[1], SRM5_OFS+OFFSIDER+_V(0,0,Stg0Longer+Stg1Longer+Stg2Longer-5.9), &srm_exhaust);
		AddExhaustStream (th_SRMs[2], SRM6_OFS+OFFSIDEL+_V(0,0,Stg0Longer+Stg1Longer+Stg2Longer-5.9), &srm_exhaust);
		AddExhaustStream (th_SRMs[3], SRM7_OFS+OFFSIDER+_V(0,0,Stg0Longer+Stg1Longer+Stg2Longer-5.9), &srm_exhaust);
		AddExhaustStream (th_SRMs[4], SRM8_OFS+OFFSIDEL+_V(0,0,Stg0Longer+Stg1Longer+Stg2Longer-5.9), &srm_exhaust);
		AddExhaustStream (th_SRMs[5], SRM9_OFS+OFFSIDER+_V(0,0,Stg0Longer+Stg1Longer+Stg2Longer-5.9), &srm_exhaust);
		thg_SRMs = CreateThrusterGroup (th_SRMs, 6, THGROUP_USER);

	}
	else {
		th_SRMs[0] = CreateThruster (SRM0_OFS+_V(0,0,Stg1Longer+Stg2Longer), _V(-SRM0_OFS.x/25,-SRM0_OFS.y/25,1), thrustvac, ph_SRMs1, Ispvac, Ispsl);
		th_SRMs[1] = CreateThruster (SRM1_OFS+_V(0,0,Stg1Longer+Stg2Longer), _V(-SRM1_OFS.x/25,-SRM1_OFS.y/25,1), thrustvac, ph_SRMs1, Ispvac, Ispsl);
		AddExhaustStream (th_SRMs[0], SRM0_OFS+_V(0,0,Stg1Longer+Stg2Longer-5.9), &srm_exhaust);
		AddExhaustStream (th_SRMs[1], SRM1_OFS+_V(0,0,Stg1Longer+Stg2Longer-5.9), &srm_exhaust);

		if (numSRMs1 == 4) {
			th_SRMs[2] = CreateThruster (SRM2_OFS+_V(0,0,Stg1Longer+Stg2Longer), _V(-SRM2_OFS.x/25,-SRM2_OFS.y/25,1), thrustvac, ph_SRMs1, Ispvac, Ispsl);
			th_SRMs[3] = CreateThruster (SRM3_OFS+_V(0,0,Stg1Longer+Stg2Longer), _V(-SRM3_OFS.x/25,-SRM3_OFS.y/25,1), thrustvac, ph_SRMs1, Ispvac, Ispsl);
			AddExhaustStream (th_SRMs[2], SRM2_OFS+_V(0,0,Stg1Longer+Stg2Longer-5.9), &srm_exhaust);
			AddExhaustStream (th_SRMs[3], SRM3_OFS+_V(0,0,Stg1Longer+Stg2Longer-5.9), &srm_exhaust);
			thg_SRMs = CreateThrusterGroup (th_SRMs, 4, THGROUP_USER);
		}
		else
			thg_SRMs = CreateThrusterGroup (th_SRMs, 2, THGROUP_USER);
	}

	CalcWeights();

}

void CVELTITAN::CalcWeights ()
{
	double tempmass;
	double SRMmass;
	if (Titan2) {
		SRMmass = MASS_CASTOR_EMPTY*(numSRMs0+numSRMs1);
	}
	else {
		SRMmass = MASS_ALGOL_EMPTY*(numSRMs0+numSRMs1);
	}
	switch (status) {

	case 0:
		tempmass = mass_fairing+mass_first+mass_srms*2.0;
		if (SecndS > 0) tempmass += mass_second_fuel+mass_second_empty;
		SetEmptyMass (tempmass + Uppermass + SRMmass + atchmass + CalculateMass());
//		if (oapiGetFocusObject() == GetHandle())
//			sprintf(oapiDebugString(), "stage %i tempmass %.3f Uppermass %.3f SRMmass %.3f atchmass %.3f CalculateMass %.3f Emptymass %.3f", status,tempmass, Uppermass, SRMmass, atchmass, CalculateMass(),GetEmptyMass());
		break;
	case 1:
		tempmass = mass_fairing+mass_first_empty;
		if (SecndS > 0) tempmass += mass_second_fuel+mass_second_empty;
		SetEmptyMass (tempmass +Uppermass + SRMmass + atchmass + CalculateMass());
//		if (oapiGetFocusObject() == GetHandle())
//			sprintf(oapiDebugString(), "stage %i tempmass %.3f Uppermass %.3f SRMmass %.3f atchmass %.3f CalculateMass %.3f Emptymass %.3f", status,tempmass, Uppermass, SRMmass, atchmass, CalculateMass(),GetEmptyMass());
		break;
	case 2:
		tempmass = mass_first_empty;
		if (SecndS > 0) tempmass += mass_second_fuel+mass_second_empty;
		SetEmptyMass (tempmass + Uppermass + SRMmass + atchmass + CalculateMass());
//		if (oapiGetFocusObject() == GetHandle())
//			sprintf(oapiDebugString(), "stage %i tempmass %.3f Uppermass %.3f SRMmass %.3f atchmass %.3f CalculateMass %.3f Emptymass %.3f", status,tempmass, Uppermass, SRMmass, atchmass, CalculateMass(),GetEmptyMass());
		break;
	case 3:
		if (SecndS > 0) SetEmptyMass (mass_second_empty + Uppermass + atchmass + CalculateMass());
		else SetEmptyMass (Uppermass + atchmass + CalculateMass());
//		if (oapiGetFocusObject() == GetHandle())
//			sprintf(oapiDebugString(), "stage %i Uppermass %.3f atchmass %.3f CalculateMass %.3f Emptymass %.3f", status, Uppermass, atchmass, CalculateMass(),GetEmptyMass());
		break;
	case 4:
		if (UprS == 1) SetEmptyMass ( TRANSTAGE_DRYMASS + atchmass + CalculateMass() );
		else if (UprS == 2) SetEmptyMass ( CENTAUR_DRYMASS + atchmass + CalculateMass() );
		else if (UprS == 3) SetEmptyMass ( AGENA_DRYMASS + atchmass + CalculateMass() );
		else if (UprS == 4) SetEmptyMass ( CHARIOT_DRYMASS + atchmass + CalculateMass() );
//		if (oapiGetFocusObject() == GetHandle())
//			sprintf(oapiDebugString(), "stage %i atchmass %.3f CalculateMass %.3f Emptymass %.3f", status, atchmass, CalculateMass(),GetEmptyMass());
		break;
	}
				
}

void CVELTITAN::SetClusterStage ()
{
	PARTICLESTREAMSPEC titan_exhaust = {
		0, .65, 120, 25.0, 0, 0.7, -.5, 0.5, PARTICLESTREAMSPEC::EMISSIVE,
			PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
		PARTICLESTREAMSPEC::ATM_FLAT, 1, 1
	};

	titan_exhaust.tex = oapiRegisterParticleTexture ("contrail_blue");

	SetSize (48);
	SetCOG_elev (27);
	SetTouchdownPoints (_V(0,-1.0,-61.0+padbias+Stg0Longer+Stg1Longer+Stg2Longer), _V(-.7,.7,-61.0+padbias+Stg0Longer+Stg1Longer+Stg2Longer), _V(.7,.7,-61.0+padbias+Stg0Longer+Stg1Longer+Stg2Longer));
	SetPMI (_V(75,75,5));
	SetCrossSections (_V(105,105,25));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.07,0.07,0.1));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
	SetLiftCoeffFunc (0); 

	if (!ph_first)  ph_first  = CreatePropellantResource (srm_prop_mass*2.0); 
//	if (!ph_main)  ph_main  = CreatePropellantResource (mass_first_fuel); 
	if (ph_second)	DelPropellantResource (ph_second);
	SetDefaultPropellantResource (ph_first); 
	ClearThrusterDefinitions();

	double ThrusterZ;
	ThrusterZ = stage0_ofs_left_flame.z;
	ThrusterZ += Stg1Longer+Stg2Longer;

	th_srb[0] = CreateThruster (_V( 2.2, 0.75,ThrusterZ), STAGE1_FLAME_DIR, thrust_main_vac, ph_first, isp_main_vac, isp_main_sl);
	th_srb[1] = CreateThruster (_V( 2.2,-0.75,ThrusterZ), STAGE1_FLAME_DIR, thrust_main_vac, ph_first, isp_main_vac, isp_main_sl);
	th_srb[2] = CreateThruster (_V(-2.2, 0.75,ThrusterZ), STAGE1_FLAME_DIR, thrust_main_vac, ph_first, isp_main_vac, isp_main_sl);
	th_srb[3] = CreateThruster (_V(-2.2,-0.75,ThrusterZ), STAGE1_FLAME_DIR, thrust_main_vac, ph_first, isp_main_vac, isp_main_sl);
	th_srb[4] = CreateThruster (_V( 0.75, 2.2,ThrusterZ), STAGE1_FLAME_DIR, thrust_main_vac, ph_first, isp_main_vac, isp_main_sl);
	th_srb[5] = CreateThruster (_V(-0.75, 2.2,ThrusterZ), STAGE1_FLAME_DIR, thrust_main_vac, ph_first, isp_main_vac, isp_main_sl);
	th_srb[6] = CreateThruster (_V( 0.75,-2.2,ThrusterZ), STAGE1_FLAME_DIR, thrust_main_vac, ph_first, isp_main_vac, isp_main_sl);
	th_srb[7] = CreateThruster (_V(-0.75,-2.2,ThrusterZ), STAGE1_FLAME_DIR, thrust_main_vac, ph_first, isp_main_vac, isp_main_sl);
	thg_main = CreateThrusterGroup (th_srb, 8, THGROUP_MAIN);

	SURFHANDLE tex_main;
	if (Titan1) tex_main = oapiRegisterExhaustTexture ("KeroExhaust");
		else tex_main = oapiRegisterExhaustTexture ("RealExhaust");
	for (int i = 0; i < 8; i++) {
			AddExhaust (th_srb[i], 25.0, 2.0, tex_main);
			AddExhaustStream(th_srb[i], &titan_exhaust);
	}


	if (status == -1) {
		SetThrusterGroupLevel (thg_srb,1.0);
//		SetThrusterGroupLevel (thg_first,1.0);
	}
	status = 0;
	stage = 0;

	CalcWeights();

	SetCameraOffset(CAMERA_OFS);
	MakeSRMs();

	ClearMeshesEx();
	PlaceComponents();
	Draw();

}

void CVELTITAN::SetSRMsStage ()
{
	if (!BoostersSet)
		SetBoosters();

	if (Cluster) {
		SetClusterStage();
		return;
	}

	SetSize (48);
	SetCOG_elev (27);
	SetTouchdownPoints (_V(0,-1.0,-41.0+padbias+Stg0Longer+Stg1Longer+Stg2Longer), _V(-.7,.7,-41.0+padbias+Stg0Longer+Stg1Longer+Stg2Longer), _V(.7,.7,-41.0+padbias+Stg0Longer+Stg1Longer+Stg2Longer));
	SetPMI (_V(75,75,5));
	SetCrossSections (_V(105,105,25));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.07,0.07,0.1));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
	SetLiftCoeffFunc (0); 

	if (!ph_first)  ph_first  = CreatePropellantResource (srm_prop_mass*2.0); 
//	if (!ph_main)  ph_main  = CreatePropellantResource (mass_first_fuel); 
	if (ph_second)	DelPropellantResource (ph_second);
	SetDefaultPropellantResource (ph_first); 
	ClearThrusterDefinitions();
	VECTOR3 ThrusterPosition, ThrusterPosition2;
	ThrusterPosition = stage0_ofs_right_flame;
	ThrusterPosition.z += Stg1Longer+Stg2Longer;
	th_srb[0] = CreateThruster (ThrusterPosition, _V(0,0,1), thrust_srm_vac, ph_first, isp_srm_vac, isp_srm_sl);
	ThrusterPosition2 = stage0_ofs_left_flame;
	ThrusterPosition2.z += Stg1Longer+Stg2Longer;
	th_srb[1] = CreateThruster (ThrusterPosition2, _V(0,0,1), thrust_srm_vac, ph_first, isp_srm_vac, isp_srm_sl);
	thg_srb = CreateThrusterGroup (th_srb, 2, THGROUP_MAIN);


	PARTICLESTREAMSPEC srb_contrail = {
		0, 8.0, 5, 150.0, 0.3, 6.0, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-8, 0.1
	};
	PARTICLESTREAMSPEC srb_exhaust = {
		0, 4.0, 20, 150.0, 0.1, 0.3, 20, 2.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-8, 0.1
	};
	AddExhaustStream (th_srb[0], ThrusterPosition2, &srb_contrail);
	AddExhaustStream (th_srb[1], ThrusterPosition, &srb_contrail);
	AddExhaustStream (th_srb[0], ThrusterPosition2, &srb_exhaust);
	AddExhaustStream (th_srb[1], ThrusterPosition, &srb_exhaust);

	if (status == -1) {
		SetThrusterGroupLevel (thg_srb,1.0);
//		SetThrusterGroupLevel (thg_first,1.0);
	}
	status = 0;
	stage = 0;

	CalcWeights();

	SetCameraOffset(CAMERA_OFS);
	MakeSRMs();

	ClearMeshesEx();
	PlaceComponents();
	Draw();


}


void CVELTITAN::SetMainStage ()
{
	PARTICLESTREAMSPEC titan_exhaust = {
		0, .65, 120, 25.0, 0, 0.7, -.5, 0.5, PARTICLESTREAMSPEC::EMISSIVE,
			PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
		PARTICLESTREAMSPEC::ATM_FLAT, 1, 1
	};

	titan_exhaust.tex = oapiRegisterParticleTexture ("contrail_blue");

	SetSize (45);
	SetCOG_elev (19);
	SetPMI (_V(75,75,5));
	SetCrossSections (_V(105,105,25));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.007,0.007,.012));
	SetTouchdownPoints (_V(0,-1.0,-38.0+padbias+Stg1Longer+Stg2Longer), _V(-.7,.7,-38.0+padbias+Stg1Longer+Stg2Longer), _V(.7,.7,-38.0+padbias+Stg1Longer+Stg2Longer));
	SetCameraOffset(CAMERA_OFS);

	status = 1;
	CalcWeights();
	ClearMeshesEx();
	PlaceComponents();
	Draw();

	ClearThrusterDefinitions();
	if (ph_first) DelPropellantResource (ph_first);
	if (ph_second)	DelPropellantResource (ph_second);
	if (!ph_main)  ph_main  = CreatePropellantResource (mass_first_fuel); 

	VECTOR3 ThrusterPosition, ThrusterPosition2;
	if (!EnginesVert) {
		ThrusterPosition = STAGE1_OFS2_FLAME;
		ThrusterPosition.z += Stg1Longer+Stg2Longer;
		ThrusterPosition2 = STAGE1_OFS1_FLAME;
		ThrusterPosition2.z += Stg1Longer+Stg2Longer;
		th_main[0] = CreateThruster (ThrusterPosition, STAGE1_FLAME_DIR, thrust_main_vac, ph_main, isp_main_vac, isp_main_sl);
		th_main[1] = CreateThruster (ThrusterPosition2, STAGE1_FLAME_DIR, thrust_main_vac, ph_main, isp_main_vac, isp_main_sl);
		thg_main = CreateThrusterGroup (th_main, 2, THGROUP_MAIN);
	}
	else {
		ThrusterPosition = T3_OFS2_FLAME;
		ThrusterPosition.z += Stg1Longer+Stg2Longer;
		ThrusterPosition2 = T3_OFS1_FLAME;
		ThrusterPosition2.z += Stg1Longer+Stg2Longer;
		th_main[0] = CreateThruster (ThrusterPosition, STAGE1_FLAME_DIR, thrust_main_vac, ph_main, isp_main_vac, isp_main_sl);
		th_main[1] = CreateThruster (ThrusterPosition2, STAGE1_FLAME_DIR, thrust_main_vac, ph_main, isp_main_vac, isp_main_sl);
		thg_main = CreateThrusterGroup (th_main, 2, THGROUP_MAIN);
	}
	SURFHANDLE tex_main;
	if (Titan1) tex_main = oapiRegisterExhaustTexture ("KeroExhaust");
		else tex_main = oapiRegisterExhaustTexture ("RealExhaust");
	for (int i = 0; i < 2; i++) AddExhaust (th_main[i], 25.0, 2.0, tex_main);
	AddExhaustStream(th_main[0], &titan_exhaust);
	AddExhaustStream(th_main[1], &titan_exhaust);

	MakeSRMs();

	if (MET > 0.0)
		SetThrusterGroupLevel (thg_main,1.0);

	SetDefaultPropellantResource (ph_main); 
//	if (ph_RCS)	DelPropellantResource (ph_RCS);

  }


void CVELTITAN::SetNoFairStage ()
{
	PARTICLESTREAMSPEC titan_exhaust = {
		0, .65, 120, 25.0, 0, 0.7, -.5, 0.5, PARTICLESTREAMSPEC::EMISSIVE,
			PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
		PARTICLESTREAMSPEC::ATM_FLAT, 1, 1
	};

	titan_exhaust.tex = oapiRegisterParticleTexture ("contrail_blue");

	SetSize (30);
	SetCOG_elev (5);
	SetPMI (_V(22.75,22.75,4.25));
	SetCrossSections (_V(100.0,100.0,78.5));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
	SetLiftCoeffFunc (0); 
	SetTouchdownPoints (_V(0,-1.0,-38.0+padbias+Stg1Longer+Stg2Longer), _V(-.7,.7,-38.0+padbias+Stg1Longer+Stg2Longer), _V(.7,.7,-38.0+padbias+Stg1Longer+Stg2Longer));
	SetCameraOffset(CAMERA_OFS);

	status = 2;
	CalcWeights();
	ClearMeshesEx();
	PlaceComponents();
	Draw();

	VECTOR3 ThrusterPosition, ThrusterPosition2;

	if (!ph_main) {
		ph_main  = CreatePropellantResource (mass_first_fuel); 

		if (!EnginesVert) {
			ThrusterPosition = STAGE1_OFS2_FLAME;
			ThrusterPosition.z += Stg1Longer+Stg2Longer;
			ThrusterPosition2 = STAGE1_OFS1_FLAME;
			ThrusterPosition2.z += Stg1Longer+Stg2Longer;
			if (invoked) {
				ThrusterPosition =  _V( 0.7,0,- STAGE1_LENGTH / 2);
				ThrusterPosition2 = _V(-0.7,0,- STAGE1_LENGTH / 2);
			}
			th_main[0] = CreateThruster (ThrusterPosition, STAGE1_FLAME_DIR, thrust_main_vac, ph_main, isp_main_vac, isp_main_sl);
			th_main[1] = CreateThruster (ThrusterPosition2, STAGE1_FLAME_DIR, thrust_main_vac, ph_main, isp_main_vac, isp_main_sl);
			thg_main = CreateThrusterGroup (th_main, 2, THGROUP_MAIN);
		}
		else {
			ThrusterPosition = T3_OFS2_FLAME;
			ThrusterPosition.z += Stg1Longer+Stg2Longer;
			ThrusterPosition2 = T3_OFS1_FLAME;
			ThrusterPosition2.z += Stg1Longer+Stg2Longer;
			th_main[0] = CreateThruster (ThrusterPosition, STAGE1_FLAME_DIR, thrust_main_vac, ph_main, isp_main_vac, isp_main_sl);
			th_main[1] = CreateThruster (ThrusterPosition2, STAGE1_FLAME_DIR, thrust_main_vac, ph_main, isp_main_vac, isp_main_sl);
			thg_main = CreateThrusterGroup (th_main, 2, THGROUP_MAIN);
		}

	SURFHANDLE tex_main;
	if (Titan1) tex_main = oapiRegisterExhaustTexture ("KeroExhaust");
		else tex_main = oapiRegisterExhaustTexture ("RealExhaust");
		
		for (int i = 0; i < 2; i++) AddExhaust (th_main[i], 25.0, 2.0, tex_main);
		AddExhaustStream(th_main[0], &titan_exhaust);
		AddExhaustStream(th_main[1], &titan_exhaust);
		if (MET > 0.0)
			SetThrusterGroupLevel (thg_main,1.0);
	}
	SetDefaultPropellantResource (ph_main); 
	if (ph_second)	DelPropellantResource (ph_second);

	MakeSRMs();

// 	if (ph_RCS)	DelPropellantResource (ph_RCS);
   
}

void CVELTITAN::SetSecondStage ()
{
	PARTICLESTREAMSPEC titan_exhaust = {
		0, .45, 120, 25.0, 0, 0.5, -.5, 0.5, PARTICLESTREAMSPEC::EMISSIVE,
			PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
		PARTICLESTREAMSPEC::ATM_FLAT, 1, 1
	};
	PARTICLESTREAMSPEC titanturb_exhaust = {
		0, .20, 120, 25.0, 0, 0.25, -.5, 0.5, PARTICLESTREAMSPEC::EMISSIVE,
			PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
		PARTICLESTREAMSPEC::ATM_FLAT, 1, 1
	};

	titan_exhaust.tex = oapiRegisterParticleTexture ("contrail_blue");
	titanturb_exhaust.tex = oapiRegisterParticleTexture ("contrail_blue");


//	bool noshift = false;
	ClearThrusterDefinitions();
	SetSize (20);
	SetCOG_elev (15);
	SetPMI (_V(23.95,23.95,3.85));
	SetCrossSections (_V(100.0,100.0,78.5));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
	SetLiftCoeffFunc (0); 
	SetTouchdownPoints (_V(0,-1.0,-18.0+padbias), _V(-.7,.7,-18.0+padbias), _V(.7,.7,-18.0+padbias));
	SetCameraOffset(CAMERA_OFS);
	
	if (ph_main) DelPropellantResource (ph_main);

	if (!ph_second)	
		if (SecndS > 0) ph_second  = CreatePropellantResource (mass_second_fuel);
		else ph_second = CreatePropellantResource (0.0);
	if (SecndS == 0) return;

//	if (RCSmass == 0) RCSmass = RCS_MASS;
//	if ((!invoked) && (ph_RCS) && (status < 3)) DelPropellantResource (ph_RCS);
//	if (!ph_RCS) ph_RCS = CreatePropellantResource (RCSmass);
	SetDefaultPropellantResource (ph_second); 


	if (GetPropellantMass(ph_second) > 0.0) 
		deploy_no_more = false;
	if (deploy_no_more) ph_second = CreatePropellantResource (0.0);

	if (invoked) {
		th_second[0] = CreateThruster (_V(0,0,-STAGE2_LENGTH/2), _V(0,0,1), thrust_second_vac, ph_second, isp_second_vac, isp_second_sl);
		th_second[1] = CreateThruster ((_V(0,0,-STAGE2_LENGTH/2)+_V(-0.9,0.0,1.0)), _V(0,0,1), 5000.0, ph_second, isp_second_vac, isp_second_sl);
	}
	else {
		th_second[0] = CreateThruster (STAGE2_OFS_FLAME+_V(0,0,Stg2EngBias+Stg2Longer), _V(0,0,1), thrust_second_vac, ph_second, isp_second_vac, isp_second_sl);
		th_second[1] = CreateThruster (STAGE2_OFS_FLAME+_V(-0.9,0.0,1.0+Stg2EngBias+Stg2Longer), _V(0,0,1), 5000.0, ph_second, isp_second_vac, isp_second_sl);
	}
	thg_second = CreateThrusterGroup (th_second, 2, THGROUP_MAIN);

	if (TitanC) {
		SURFHANDLE tex_main = oapiRegisterExhaustTexture ("Exhaust_atsme");

		AddExhaust (th_second[0], 20.0, 2.0, tex_main);
		AddExhaust (th_second[1], 4.0, 0.4, tex_main);
		AddExhaustStream(th_second[0], &titan_exhaust);
		AddExhaustStream(th_second[1], &titanturb_exhaust);
	}
	else {
		SURFHANDLE tex_main;
		if (Titan1) tex_main = oapiRegisterExhaustTexture ("KeroExhaust");
			else tex_main = oapiRegisterExhaustTexture ("RealExhaust");
		AddExhaust (th_second[0], 15.0, 1.0, tex_main);
		AddExhaust (th_second[1], 4.0, 0.4, tex_main);
		AddExhaustStream(th_second[0], &titan_exhaust);
		AddExhaustStream(th_second[1], &titanturb_exhaust);
	}

	if (status < 3) SetThrusterGroupLevel (thg_second,1.0);
//	SetAttControlsSecond();
	SetAttitudeRotLevel(_V(0,0,0));
	pitchinput = 0.0;
	yawinput = 0.0;
	rollinput = 0.0;
	status = 3;
	stage = 3;


	CalcWeights();
	ClearMeshesEx();
	PlaceComponents();
//	if (!noshift) Draw(_V(0,0,-10.0));
//		else Draw(_V(0,0,0));
	Draw();

	if (SecondStageRCS) {
		THRUSTER_HANDLE th_rot[2];
		th_rot[0] = CreateThruster (_V(0,0, 5), _V(0, 1,0), MAX_ATT_TRANSTAGE, ph_second, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(0,0,-5), _V(0,-1,0), MAX_ATT_TRANSTAGE, ph_second, ISP_TRANSTAGE_SL);

		thg_pitchup=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHUP);

		th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,1,0),MAX_ATT_TRANSTAGE, ph_second, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,1,0),MAX_ATT_TRANSTAGE, ph_second, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_UP);
		th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,-1,0), MAX_ATT_TRANSTAGE, ph_second, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,-1,0), MAX_ATT_TRANSTAGE, ph_second, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_DOWN);

		th_rot[0] = CreateThruster (_V(0,0, 5), _V(0,-1,0), MAX_ATT_TRANSTAGE, ph_second, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(0,0,-5), _V(0, 1,0), MAX_ATT_TRANSTAGE, ph_second, ISP_TRANSTAGE_SL);

		thg_pitchdown=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHDOWN);

		th_rot[0] = CreateThruster (_V(0,0, 4), _V(-1,0,0), MAX_ATT_TRANSTAGE, ph_second, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(0,0,-4), _V( 1,0,0), MAX_ATT_TRANSTAGE, ph_second, ISP_TRANSTAGE_SL);

		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWLEFT);

		th_rot[0] = CreateThruster (_V(0,0, 4), _V( 1,0,0), MAX_ATT_TRANSTAGE, ph_second, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(0,0,-4), _V(-1,0,0), MAX_ATT_TRANSTAGE, ph_second, ISP_TRANSTAGE_SL);

		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWRIGHT);

		th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(-1,0,0), MAX_ATT_TRANSTAGE, ph_second, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 1, THGROUP_ATT_LEFT);
		th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(1,0,0),MAX_ATT_TRANSTAGE, ph_second, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 1, THGROUP_ATT_RIGHT);

		th_rot[0] = CreateThruster (_V(13,0,0), _V(0,-1,0), MAX_ATT_TRANSTAGE, ph_second, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, 1,0), MAX_ATT_TRANSTAGE, ph_second, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKRIGHT);

		th_rot[0] = CreateThruster (_V(13,0, 0), _V(0, 1,0), MAX_ATT_TRANSTAGE, ph_second, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, -1,0), MAX_ATT_TRANSTAGE, ph_second, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKLEFT);

		th_rot[0] = CreateThruster (_V(0,0, -4), _V(0,0,1), MAX_ATT_TRANSTAGE, ph_second, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 1, THGROUP_ATT_FORWARD);
		th_rot[0] = CreateThruster (_V(0,0, 4), _V(0,0,-1), MAX_ATT_TRANSTAGE, ph_second, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 1, THGROUP_ATT_BACK);
	}

}

void CVELTITAN::SetUpperStage()
{

	SetCameraOffset(CAMERA_OFS);
	if (UprS == 1)
	{

//		const VECTOR3 VECTOR_ZERO = _V(0,0,0);
		SURFHANDLE tex = oapiRegisterExhaustTexture ("RealExhaust");

		/* CVE-lite function ! */
		/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
		/* with the COMMON_MESH class (not used here) */


		ClearThrusterDefinitions();
/*
		PARTICLESTREAMSPEC contrail = 
		{
			0, 25.0, 7, 350, 0.3, 4.0, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
			PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
			PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
		};

		PARTICLESTREAMSPEC exhaust_main = {
			0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
			PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
			PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
		};
		exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");
*/
		SetSize (8);

		/* CVE-lite function ! */
		/* IMPORTANT */
		/* double CalculateMass() will return the all-up mass of the combined payloads and
		/* upper stages. Simply add this to your normal dry mass. */
		
		/* always add payload mass [CalculateMass()] to the vessel's empty mass */

		
		SetTouchdownPoints (_V(0,-1.0,-3), _V(-.7,.7,-3), _V(.7,.7,-3));
		SetCOG_elev (5);
		SetPMI (_V(17,17,1.5));
		SetCrossSections (_V(4,4,2.5));
		SetCW (0.1, 0.3, 1.4, 1.4);
		SetRotDrag (_V(0.7,0.7,1.2));
		SetPitchMomentScale (1e-6);
		SetBankMomentScale (1e-6);
		SetLiftCoeffFunc (0); 



	

		if (ph_second) DelPropellantResource (ph_second);

		/* verify that the propellent hasn't been created before making it */
		if (ph_upper==NULL)  
			ph_upper  = CreatePropellantResource(TRANSTAGE_PROP_MASS); 


		SetDefaultPropellantResource (ph_upper); 

		th_upper[0] = CreateThruster ( _V(0,0.72,-4.8), _V(0,0,1), THRUST_TRANSTAGE_VAC, ph_upper, ISP_TRANSTAGE_VAC, ISP_TRANSTAGE_SL);
	//	AddExhaustStream (th_upper[0], _V(0.72,0,-7), &contrail);
	//	AddExhaustStream (th_upper[0], _V(0.72,0,-7), &exhaust_main);

		th_upper[1] = CreateThruster ( _V(0,-0.72,-4.8), _V(0,0,1), THRUST_TRANSTAGE_VAC, ph_upper, ISP_TRANSTAGE_VAC, ISP_TRANSTAGE_SL);
	//	AddExhaustStream (th_upper[1], _V(-0.72,0,-7), &contrail);
	//	AddExhaustStream (th_upper[1], _V(-0.72,0,-7), &exhaust_main);

		thg_upper = CreateThrusterGroup (th_upper, 2, THGROUP_MAIN);
		
		AddExhaust (th_upper[0], 8.0, 0.9, tex);
		AddExhaust (th_upper[1], 8.0, 0.9, tex);

		EnableTransponder(true);
 
		SetThrusterGroupLevel (thg_upper,0.0);
		if (status < 4) SetThrusterGroupLevel (thg_upper,1.0);

		status = 4;
		stage = 4;
		CalcWeights();
		SetAttitudeRotLevel(_V(0,0,0));
		pitchinput = 0.0;
		yawinput = 0.0;
		rollinput = 0.0;
		ClearMeshesEx();
		PlaceComponents();
		Draw();


		THRUSTER_HANDLE th_rot[2];
		th_rot[0] = CreateThruster (_V(0,0, 5), _V(0, 1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(0,0,-5), _V(0,-1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);

		thg_pitchup=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHUP);

		th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,1,0),MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,1,0),MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_UP);
		th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,-1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,-1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_DOWN);

		th_rot[0] = CreateThruster (_V(0,0, 5), _V(0,-1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(0,0,-5), _V(0, 1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);

		thg_pitchdown=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHDOWN);

		th_rot[0] = CreateThruster (_V(0,0, 4), _V(-1,0,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(0,0,-4), _V( 1,0,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);

		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWLEFT);

		th_rot[0] = CreateThruster (_V(0,0, 4), _V( 1,0,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(0,0,-4), _V(-1,0,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);

		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWRIGHT);

		th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(-1,0,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 1, THGROUP_ATT_LEFT);
		th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(1,0,0),MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 1, THGROUP_ATT_RIGHT);

		th_rot[0] = CreateThruster (_V(13,0,0), _V(0,-1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, 1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKRIGHT);

		th_rot[0] = CreateThruster (_V(13,0, 0), _V(0, 1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, -1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKLEFT);

		th_rot[0] = CreateThruster (_V(0,0, -4), _V(0,0,1), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 1, THGROUP_ATT_FORWARD);
		th_rot[0] = CreateThruster (_V(0,0, 4), _V(0,0,-1), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 1, THGROUP_ATT_BACK);

	}
	else if (UprS == 2)
	{

	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearThrusterDefinitions();

	SURFHANDLE tex = oapiRegisterExhaustTexture("Exhaust_atsme");


/*	PARTICLESTREAMSPEC contrail = 
	{
		0, 25.0, 7, 350, 0.3, 4.0, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");
*/
	SetSize (8);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */


	SetTouchdownPoints (_V(0,-1.0,-3), _V(-.7,.7,-3), _V(.7,.7,-3));
	SetPMI (_V(17,17,1.5));
	SetCOG_elev (5);
	SetCrossSections (_V(4,4,2.5));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
	SetLiftCoeffFunc (0); 

	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */


	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	if (ph_second) DelPropellantResource (ph_second);

	/* verify that the propellent hasn't been created before making it */
	if (ph_upper==NULL)  
		ph_upper  = CreatePropellantResource(CENTAUR_PROP_MASS); 


	SetDefaultPropellantResource (ph_upper); 

	th_upper[0] = CreateThruster ( _V(0,0.72,-4.5), _V(0,0,1), THRUST_CENTAUR_VAC, ph_upper, ISP_CENTAUR_VAC, ISP_CENTAUR_SL);
//	AddExhaustStream (th_upper[0], _V(0.72,0,-7), &contrail);
//	AddExhaustStream (th_upper[0], _V(0.72,0,-7), &exhaust_main);

	th_upper[1] = CreateThruster ( _V(0,-0.72,-4.5), _V(0,0,1), THRUST_CENTAUR_VAC, ph_upper, ISP_CENTAUR_VAC, ISP_CENTAUR_SL);
//	AddExhaustStream (th_upper[1], _V(-0.72,0,-7), &contrail);
//	AddExhaustStream (th_upper[1], _V(-0.72,0,-7), &exhaust_main);

	thg_upper = CreateThrusterGroup (th_upper, 2, THGROUP_MAIN);
	
	AddExhaust (th_upper[0], 8.0, 0.9, tex);
	AddExhaust (th_upper[1], 8.0, 0.9, tex);

	THRUSTER_HANDLE th_rot[2];
	th_rot[0] = CreateThruster (_V(0,0, 5), _V(0, 1,0), MAX_ATT_CENTAUR, ph_upper, ISP_CENTAUR_SL);
	th_rot[1] = CreateThruster (_V(0,0,-5), _V(0,-1,0), MAX_ATT_CENTAUR, ph_upper, ISP_CENTAUR_SL);

	thg_pitchup=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHUP);

	th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,1,0),MAX_ATT_CENTAUR, ph_upper, ISP_CENTAUR_SL);
	th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,1,0),MAX_ATT_CENTAUR, ph_upper, ISP_CENTAUR_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_UP);
	th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,-1,0), MAX_ATT_CENTAUR, ph_upper, ISP_CENTAUR_SL);
	th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,-1,0), MAX_ATT_CENTAUR, ph_upper, ISP_CENTAUR_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_DOWN);

	th_rot[0] = CreateThruster (_V(0,0, 5), _V(0,-1,0), MAX_ATT_CENTAUR, ph_upper, ISP_CENTAUR_SL);
	th_rot[1] = CreateThruster (_V(0,0,-5), _V(0, 1,0), MAX_ATT_CENTAUR, ph_upper, ISP_CENTAUR_SL);

	thg_pitchdown=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHDOWN);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V(-1,0,0), MAX_ATT_CENTAUR, ph_upper, ISP_CENTAUR_SL);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V( 1,0,0), MAX_ATT_CENTAUR, ph_upper, ISP_CENTAUR_SL);

	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWLEFT);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V( 1,0,0), MAX_ATT_CENTAUR, ph_upper, ISP_CENTAUR_SL);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V(-1,0,0), MAX_ATT_CENTAUR, ph_upper, ISP_CENTAUR_SL);

	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWRIGHT);

	th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(-1,0,0), MAX_ATT_CENTAUR, ph_upper, ISP_CENTAUR_SL);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_LEFT);
	th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(1,0,0),MAX_ATT_CENTAUR, ph_upper, ISP_CENTAUR_SL);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_RIGHT);

	th_rot[0] = CreateThruster (_V(13,0,0), _V(0,-1,0), MAX_ATT_CENTAUR, ph_upper, ISP_CENTAUR_SL);
	th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, 1,0), MAX_ATT_CENTAUR, ph_upper, ISP_CENTAUR_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKRIGHT);

	th_rot[0] = CreateThruster (_V(13,0, 0), _V(0, 1,0), MAX_ATT_CENTAUR, ph_upper, ISP_CENTAUR_SL);
	th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, -1,0), MAX_ATT_CENTAUR, ph_upper, ISP_CENTAUR_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKLEFT);

	th_rot[0] = CreateThruster (_V(0,0, -4), _V(0,0,1), MAX_ATT_CENTAUR, ph_upper, ISP_CENTAUR_SL);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_FORWARD);
	th_rot[0] = CreateThruster (_V(0,0, 4), _V(0,0,-1), MAX_ATT_CENTAUR, ph_upper, ISP_CENTAUR_SL);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_BACK);

	EnableTransponder(true);

		SetThrusterGroupLevel (thg_upper,0.0);
		if (status < 4) SetThrusterGroupLevel (thg_upper,1.0);

		SetAttitudeRotLevel(_V(0,0,0));
		pitchinput = 0.0;
		yawinput = 0.0;
		rollinput = 0.0;
		status = 4;
		stage = 4;
		CalcWeights();

		ClearMeshesEx();
		PlaceComponents();
		Draw();

	}
	else if (UprS == 3)
	{

//		const VECTOR3 VECTOR_ZERO = _V(0,0,0);
		SURFHANDLE tex = oapiRegisterExhaustTexture ("RealExhaust");

		/* CVE-lite function ! */
		/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
		/* with the COMMON_MESH class (not used here) */


		ClearThrusterDefinitions();
/*
		PARTICLESTREAMSPEC contrail = 
		{
			0, 25.0, 7, 350, 0.3, 4.0, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
			PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
			PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
		};

		PARTICLESTREAMSPEC exhaust_main = {
			0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
			PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
			PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
		};
		exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");
*/
		SetSize (8);

		/* CVE-lite function ! */
		/* IMPORTANT */
		/* double CalculateMass() will return the all-up mass of the combined payloads and
		/* upper stages. Simply add this to your normal dry mass. */
		
		/* always add payload mass [CalculateMass()] to the vessel's empty mass */


		SetTouchdownPoints (_V(0,-1.0,-3), _V(-.7,.7,-3), _V(.7,.7,-3));
		SetCOG_elev (5);
		SetPMI (_V(17,17,1.5));
		SetCrossSections (_V(4,4,2.5));
		SetCW (0.1, 0.3, 1.4, 1.4);
		SetRotDrag (_V(0.7,0.7,1.2));
		SetPitchMomentScale (1e-6);
		SetBankMomentScale (1e-6);
		SetLiftCoeffFunc (0); 



	

		if (ph_second) DelPropellantResource (ph_second);

		/* verify that the propellent hasn't been created before making it */
		if (ph_upper==NULL)  
			ph_upper  = CreatePropellantResource(AGENA_PROP_MASS); 


		SetDefaultPropellantResource (ph_upper); 

		th_upper[0] = CreateThruster ( _V(0,0.0,-4.8), _V(0,0,1), THRUST_AGENA_VAC, ph_upper, ISP_AGENA_VAC, ISP_AGENA_SL);
	//	AddExhaustStream (th_upper[0], _V(0.72,0,-7), &contrail);
	//	AddExhaustStream (th_upper[0], _V(0.72,0,-7), &exhaust_main);

		thg_upper = CreateThrusterGroup (th_upper, 1, THGROUP_MAIN);
		
		AddExhaust (th_upper[0], 8.0, 0.9, tex);

		EnableTransponder(true);
 
		SetThrusterGroupLevel (thg_upper,0.0);
		if (status < 4) SetThrusterGroupLevel (thg_upper,1.0);

		status = 4;
		stage = 4;
		CalcWeights();
		SetAttitudeRotLevel(_V(0,0,0));
		pitchinput = 0.0;
		yawinput = 0.0;
		rollinput = 0.0;
		ClearMeshesEx();
		PlaceComponents();
		Draw();


		THRUSTER_HANDLE th_rot[2];
		th_rot[0] = CreateThruster (_V(0,0, 5), _V(0, 1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(0,0,-5), _V(0,-1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);

		thg_pitchup=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHUP);

		th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,1,0),MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,1,0),MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_UP);
		th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,-1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,-1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_DOWN);

		th_rot[0] = CreateThruster (_V(0,0, 5), _V(0,-1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(0,0,-5), _V(0, 1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);

		thg_pitchdown=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHDOWN);

		th_rot[0] = CreateThruster (_V(0,0, 4), _V(-1,0,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(0,0,-4), _V( 1,0,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);

		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWLEFT);

		th_rot[0] = CreateThruster (_V(0,0, 4), _V( 1,0,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(0,0,-4), _V(-1,0,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);

		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWRIGHT);

		th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(-1,0,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 1, THGROUP_ATT_LEFT);
		th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(1,0,0),MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 1, THGROUP_ATT_RIGHT);

		th_rot[0] = CreateThruster (_V(13,0,0), _V(0,-1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, 1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKRIGHT);

		th_rot[0] = CreateThruster (_V(13,0, 0), _V(0, 1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, -1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKLEFT);

		th_rot[0] = CreateThruster (_V(0,0, -4), _V(0,0,1), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 1, THGROUP_ATT_FORWARD);
		th_rot[0] = CreateThruster (_V(0,0, 4), _V(0,0,-1), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 1, THGROUP_ATT_BACK);

	}

	else if (UprS == 4)
	{

//		const VECTOR3 VECTOR_ZERO = _V(0,0,0);
		SURFHANDLE tex = oapiRegisterExhaustTexture ("FluorineExhaust");

		/* CVE-lite function ! */
		/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
		/* with the COMMON_MESH class (not used here) */


		ClearThrusterDefinitions();
/*
		PARTICLESTREAMSPEC contrail = 
		{
			0, 25.0, 7, 350, 0.3, 4.0, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
			PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
			PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
		};

		PARTICLESTREAMSPEC exhaust_main = {
			0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
			PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
			PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
		};
		exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");
*/
		SetSize (8);

		/* CVE-lite function ! */
		/* IMPORTANT */
		/* double CalculateMass() will return the all-up mass of the combined payloads and
		/* upper stages. Simply add this to your normal dry mass. */
		
		/* always add payload mass [CalculateMass()] to the vessel's empty mass */

		
		SetTouchdownPoints (_V(0,-1.0,-3), _V(-.7,.7,-3), _V(.7,.7,-3));
		SetCOG_elev (5);
		SetPMI (_V(17,17,1.5));
		SetCrossSections (_V(4,4,2.5));
		SetCW (0.1, 0.3, 1.4, 1.4);
		SetRotDrag (_V(0.7,0.7,1.2));
		SetPitchMomentScale (1e-6);
		SetBankMomentScale (1e-6);
		SetLiftCoeffFunc (0); 



	

		if (ph_second) DelPropellantResource (ph_second);

		/* verify that the propellent hasn't been created before making it */
		if (ph_upper==NULL)  
			ph_upper  = CreatePropellantResource(CHARIOT_PROP_MASS); 


		SetDefaultPropellantResource (ph_upper); 

		th_upper[0] = CreateThruster ( _V(0,0.0,-4.94), _V(0,0,1), THRUST_CHARIOT_VAC, ph_upper, ISP_CHARIOT_VAC, ISP_CHARIOT_SL);
	//	AddExhaustStream (th_upper[0], _V(0.72,0,-7), &contrail);
	//	AddExhaustStream (th_upper[0], _V(0.72,0,-7), &exhaust_main);

		thg_upper = CreateThrusterGroup (th_upper, 1, THGROUP_MAIN);
		
		AddExhaust (th_upper[0], 11.0, 1.1, tex);

		EnableTransponder(true);
 
		SetThrusterGroupLevel (thg_upper,0.0);
		if (status < 4) SetThrusterGroupLevel (thg_upper,1.0);

		status = 4;
		stage = 4;
		CalcWeights();
		SetAttitudeRotLevel(_V(0,0,0));
		pitchinput = 0.0;
		yawinput = 0.0;
		rollinput = 0.0;
		ClearMeshesEx();
		PlaceComponents();
		Draw();


		THRUSTER_HANDLE th_rot[2];
		th_rot[0] = CreateThruster (_V(0,0, 5), _V(0, 1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(0,0,-5), _V(0,-1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);

		thg_pitchup=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHUP);

		th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,1,0),MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,1,0),MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_UP);
		th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,-1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,-1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_DOWN);

		th_rot[0] = CreateThruster (_V(0,0, 5), _V(0,-1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(0,0,-5), _V(0, 1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);

		thg_pitchdown=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHDOWN);

		th_rot[0] = CreateThruster (_V(0,0, 4), _V(-1,0,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(0,0,-4), _V( 1,0,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);

		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWLEFT);

		th_rot[0] = CreateThruster (_V(0,0, 4), _V( 1,0,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(0,0,-4), _V(-1,0,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);

		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWRIGHT);

		th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(-1,0,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 1, THGROUP_ATT_LEFT);
		th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(1,0,0),MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 1, THGROUP_ATT_RIGHT);

		th_rot[0] = CreateThruster (_V(13,0,0), _V(0,-1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, 1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKRIGHT);

		th_rot[0] = CreateThruster (_V(13,0, 0), _V(0, 1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, -1,0), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKLEFT);

		th_rot[0] = CreateThruster (_V(0,0, -4), _V(0,0,1), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 1, THGROUP_ATT_FORWARD);
		th_rot[0] = CreateThruster (_V(0,0, 4), _V(0,0,-1), MAX_ATT_TRANSTAGE, ph_upper, ISP_TRANSTAGE_SL);
		CreateThrusterGroup (th_rot, 1, THGROUP_ATT_BACK);

	}

}

void CVELTITAN::SeparateStage (UINT stage)
{
	
	char name1[256], name2[256], name3[256];
	VESSELSTATUS vs;
	VESSELSTATUS vs1;
	VESSELSTATUS vs2;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(0,0,0);
	VECTOR3 ofs1 = _V(0,0,0);
	VECTOR3 vel1 = _V(0,0,0);
    VECTOR3 ofs2 = _V(0,0,0);
	VECTOR3 vel2 = _V(0,0,0);
	GetStatus (vs);
	GetStatus (vs1);
	GetStatus (vs2);
	pitchinput=0.0;
	yawinput=0.0;
	rollinput=0.0;
	bjustseparated=true;

	if (stage == -1)
	{
		ofs1 = OFS_GEM1;
		ofs2 = OFS_GEM2;
		vel1 = _V(0.0,-4.0, 0.0);
		vel2 = _V(0.0, -4.0, 0.0);
	}
	
	else if (stage == 0)
	{
		ofs1 = stage0_ofs_right;
		ofs2 = stage0_ofs_left;
		ofs1.z += Stg1Longer+Stg2Longer;
		ofs2.z += Stg1Longer+Stg2Longer;
		vel1 = _V(2.0,0, 0.0);
		vel2 = _V(-2.0, 0, 0.0);
		if (Cluster) vel2 = _V(0,0,0);
	}
	
	else if (stage == 1)
	{
		ofs1 = ofs_shroud1;		
		vel1 = _V(4.0,2.0,0.2); 
		ofs2 = ofs_shroud2;		
		vel2 = _V(-4.0,2.0,0.2);
		ofs = ofs_shroud3;		
		vel = _V(0.0,-5.0,0.2);
	}
	else if (stage == 2)
	{
		ofs1 = STAGE1_OFS;	
		vel1 = _V(0,0,-2.0);		
		ofs1.z += Stg1Longer + Stg2Longer + Stg1Bias;
		ofs2 = MID_OFS;	
		ofs2.z += Stg2Longer;
		vel2 = _V(0,0,-1.0);	

	}

	else if (stage == 3)
	{
		ofs1 = STAGE2_OFS;
		vel1 = _V(0,0,-2.0);	
		ofs1.z += Stg2Bias + Stg2Longer;	
		ofs2 = INTERSTAGE2_OFS;	
		vel2 = _V(0,0,-2.0);	

	}

	VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
	VECTOR3 rofs1, rvel1 = {vs1.rvel.x, vs1.rvel.y, vs1.rvel.z};
	VECTOR3 rofs2, rvel2 = {vs2.rvel.x, vs2.rvel.y, vs2.rvel.z};
	Local2Rel (ofs, vs.rpos);
	Local2Rel (ofs1, vs1.rpos);
	Local2Rel (ofs2, vs2.rpos);
	GlobalRot (vel, rofs);
	GlobalRot (vel1, rofs1);
	GlobalRot (vel2, rofs2);

	vs.rvel.x = rvel.x+rofs.x;
	vs.rvel.y = rvel.y+rofs.y;
	vs.rvel.z = rvel.z+rofs.z;

	vs1.rvel.x = rvel1.x+rofs1.x;
	vs1.rvel.y = rvel1.y+rofs1.y;
	vs1.rvel.z = rvel1.z+rofs1.z;
	vs2.rvel.x = rvel2.x+rofs2.x;
	vs2.rvel.y = rvel2.y+rofs2.y;
	vs2.rvel.z = rvel2.z+rofs2.z;

	
	strcpy (name1, GetName());
	strcpy (name2, GetName());
	strcpy (name3, GetName());

	if ((stage == 0) && (Cluster))
	{
		vs2.vrot.x = 0.0;
		vs2.vrot.y = 0.0;
		vs2.vrot.z = 0.0;
		oapiCreateVessel(strcat (name2, " Cluster Stage"), "CVELTitan/Titan_Cluster_Stage", vs2);
		oapiCreateVessel(strcat (name3, " Cluster Stage Interstage"), "CVELTitan/Titan_Cluster_Interstage", vs2);
		DelPropellantResource (ph_first);

		DelThrusterGroup (thg_first, THGROUP_MAIN, true);
		DelThrusterGroup (thg_pitchup, THGROUP_ATT_PITCHUP, true);
		DelThrusterGroup (thg_pitchdown, THGROUP_ATT_PITCHDOWN, true);

		DelThrusterGroup (THGROUP_ATT_BANKLEFT, true);
		DelThrusterGroup (THGROUP_ATT_BANKRIGHT, true);
		DelThrusterGroup (THGROUP_ATT_YAWLEFT, true);
		DelThrusterGroup (THGROUP_ATT_YAWRIGHT, true);

		ClearThrusterDefinitions();
		if (fairing > 0)
			SetMainStage ();	
		else SetNoFairStage();
		bsolidsfired = false;
		PlayVesselWave3(MyID,PLAYBANG);
	}
	else if (stage == 0)
	{
	PARTICLESTREAMSPEC contrail = 
	{
//		0, 0.25, 10.0, 9.0, 0.5, 1.0, 10, 3.0, PARTICLESTREAMSPEC::EMISSIVE,
		0, 0.75, 29, 50.0, 0.5, 0.1, 7, 3.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0.1, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 0, 0
	};
		vs1.vrot.x = 0.0;
		vs1.vrot.y = -0.08;
		vs1.vrot.z = 0.0;		
		vs2.vrot.x = 0.0;
		vs2.vrot.y = 0.08;
		vs2.vrot.z = 0.0;
		if (SRM_Type == UA1207) {
			hanSRM1 = oapiCreateVessel(strcat (name1, " SRB 1"), "CVELTitan/UA1207W", vs1);
			hanSRM2 = oapiCreateVessel(strcat (name2, " SRB 2"), "CVELTitan/UA1207E", vs2);
			SRM1 = oapiGetVesselInterface(hanSRM1);
			SRM2 = oapiGetVesselInterface(hanSRM2);
			ph_SRM1  = SRM1->CreatePropellantResource(100,100,1.0); 
			th_sep[0] = SRM1->CreateThruster ( _V(-1.1, 0.727, 16.0), _V( 0.707,-0.707,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[1] = SRM1->CreateThruster ( _V(-1.1, 0.727, 15.5), _V( 0.707,-0.707,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[2] = SRM1->CreateThruster ( _V(-1.1, 0.727, 15.0), _V( 0.707,-0.707,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[3] = SRM1->CreateThruster ( _V(-1.45, 1.45,-12.75), _V( 0.707,-0.707,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[4] = SRM1->CreateThruster ( _V(-1.45, 1.45,-13.25), _V( 0.707,-0.707,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[5] = SRM1->CreateThruster ( _V(-1.45, 1.45,-13.75), _V( 0.707,-0.707,0), 3500, ph_SRM1, 1e3, 1e3);
			SRM1->AddExhaustStream (th_sep[0], _V(-1.1, 0.727, 16.0), &contrail);
			SRM1->AddExhaustStream (th_sep[1], _V(-1.1, 0.727, 15.5), &contrail);
			SRM1->AddExhaustStream (th_sep[2], _V(-1.1, 0.727, 15.0), &contrail);
			SRM1->AddExhaustStream (th_sep[3], _V(-1.45, 1.45,-12.75), &contrail);
			SRM1->AddExhaustStream (th_sep[4], _V(-1.45, 1.45,-13.25), &contrail);
			SRM1->AddExhaustStream (th_sep[5], _V(-1.45, 1.45,-13.75), &contrail);
			SRM1->CreateThrusterGroup (th_sep, 6, THGROUP_MAIN);
			SRM1->SetThrusterGroupLevel(THGROUP_MAIN,1.0);

			ph_SRM2  = SRM2->CreatePropellantResource(100,100,1.0); 
			th_sep[0] = SRM2->CreateThruster ( _V( 1.1,-0.727, 16.0), _V(-0.707, 0.707,0), 3500, ph_SRM2, 1e3, 1e3);
			th_sep[1] = SRM2->CreateThruster ( _V( 1.1,-0.727, 15.5), _V(-0.707, 0.707,0), 3500, ph_SRM2, 1e3, 1e3);
			th_sep[2] = SRM2->CreateThruster ( _V( 1.1,-0.727, 15.0), _V(-0.707, 0.707,0), 3500, ph_SRM2, 1e3, 1e3);
			th_sep[3] = SRM2->CreateThruster ( _V( 1.45,-1.45,-12.75), _V(-0.707, 0.707,0), 3500, ph_SRM2, 1e3, 1e3);
			th_sep[4] = SRM2->CreateThruster ( _V( 1.45,-1.45,-13.25), _V(-0.707, 0.707,0), 3500, ph_SRM2, 1e3, 1e3);
			th_sep[5] = SRM2->CreateThruster ( _V( 1.45,-1.45,-13.75), _V(-0.707, 0.707,0), 3500, ph_SRM2, 1e3, 1e3);
			SRM2->AddExhaustStream (th_sep[0], _V( 1.1,-0.727, 16.0), &contrail);
			SRM2->AddExhaustStream (th_sep[1], _V( 1.1,-0.727, 15.5), &contrail);
			SRM2->AddExhaustStream (th_sep[2], _V( 1.1,-0.727, 15.0), &contrail);
			SRM2->AddExhaustStream (th_sep[3], _V( 1.45,-1.45,-12.75), &contrail);
			SRM2->AddExhaustStream (th_sep[4], _V( 1.45,-1.45,-13.25), &contrail);
			SRM2->AddExhaustStream (th_sep[5], _V( 1.45,-1.45,-13.75), &contrail);
			SRM2->CreateThrusterGroup (th_sep, 6, THGROUP_MAIN);
			SRM2->SetThrusterGroupLevel(THGROUP_MAIN,1.0);
		}
		else if (SRM_Type == UA1003) {
			hanSRM1 = oapiCreateVessel(strcat (name1, " SRB 1"), "CVELTitan/UA1003W", vs1);
			hanSRM2 = oapiCreateVessel(strcat (name2, " SRB 2"), "CVELTitan/UA1003E", vs2);
			SRM1 = oapiGetVesselInterface(hanSRM1);
			SRM2 = oapiGetVesselInterface(hanSRM2);
			ph_SRM1  = SRM1->CreatePropellantResource(100,100,1.0); 
			th_sep[0] = SRM1->CreateThruster ( _V(-1.1, 0.727, 19.0), _V( 0.707,-0.707,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[1] = SRM1->CreateThruster ( _V(-1.1, 0.727, 19.0), _V( 0.707,-0.707,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[2] = SRM1->CreateThruster ( _V(-1.45, 1.45,-13.25), _V( 0.707,-0.707,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[3] = SRM1->CreateThruster ( _V(-1.45, 1.45,-13.75), _V( 0.707,-0.707,0), 3500, ph_SRM1, 1e3, 1e3);
			SRM1->AddExhaustStream (th_sep[0], _V(-1.1, 0.727, 3.80), &contrail);
			SRM1->AddExhaustStream (th_sep[1], _V(-1.1, 0.727, 4.25), &contrail);
			SRM1->AddExhaustStream (th_sep[2], _V(-1.45, 1.45,-13.25), &contrail);
			SRM1->AddExhaustStream (th_sep[3], _V(-1.45, 1.45,-13.75), &contrail);
			SRM1->CreateThrusterGroup (th_sep, 4, THGROUP_MAIN);
			SRM1->SetThrusterGroupLevel(THGROUP_MAIN,1.0);

			ph_SRM2  = SRM2->CreatePropellantResource(100,100,1.0); 
			th_sep[0] = SRM2->CreateThruster ( _V( 1.1,-0.727, 19.0), _V(-0.707, 0.707,0), 3500, ph_SRM2, 1e3, 1e3);
			th_sep[1] = SRM2->CreateThruster ( _V( 1.1,-0.727, 19.0), _V(-0.707, 0.707,0), 3500, ph_SRM2, 1e3, 1e3);
			th_sep[2] = SRM2->CreateThruster ( _V( 1.45,-1.45,-13.25), _V(-0.707, 0.707,0), 3500, ph_SRM2, 1e3, 1e3);
			th_sep[3] = SRM2->CreateThruster ( _V( 1.45,-1.45,-13.75), _V(-0.707, 0.707,0), 3500, ph_SRM2, 1e3, 1e3);
			SRM2->AddExhaustStream (th_sep[0], _V( 1.1,-0.727, 3.80), &contrail);
			SRM2->AddExhaustStream (th_sep[1], _V( 1.1,-0.727, 4.25), &contrail);
			SRM2->AddExhaustStream (th_sep[2], _V( 1.45,-1.45,-13.25), &contrail);
			SRM2->AddExhaustStream (th_sep[3], _V( 1.45,-1.45,-13.75), &contrail);
			SRM2->CreateThrusterGroup (th_sep, 4, THGROUP_MAIN);
			SRM2->SetThrusterGroupLevel(THGROUP_MAIN,1.0);
		}
		else if (SRM_Type == UA1208) {
			hanSRM1 = oapiCreateVessel(strcat (name1, " SRB 1"), "CVELTitan/UA1208W", vs1);
			hanSRM2 = oapiCreateVessel(strcat (name2, " SRB 2"), "CVELTitan/UA1208E", vs2);
			SRM1 = oapiGetVesselInterface(hanSRM1);
			SRM2 = oapiGetVesselInterface(hanSRM2);
			ph_SRM1  = SRM1->CreatePropellantResource(100,100,1.0); 
			th_sep[0] = SRM1->CreateThruster ( _V(-1.1, 0.727, 19.0), _V( 0.707,-0.707,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[1] = SRM1->CreateThruster ( _V(-1.1, 0.727, 18.5), _V( 0.707,-0.707,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[2] = SRM1->CreateThruster ( _V(-1.1, 0.727, 18.0), _V( 0.707,-0.707,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[3] = SRM1->CreateThruster ( _V(-1.45, 1.45,-12.75), _V( 0.707,-0.707,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[4] = SRM1->CreateThruster ( _V(-1.45, 1.45,-13.25), _V( 0.707,-0.707,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[5] = SRM1->CreateThruster ( _V(-1.45, 1.45,-13.75), _V( 0.707,-0.707,0), 3500, ph_SRM1, 1e3, 1e3);
			SRM1->AddExhaustStream (th_sep[0], _V(-1.1, 0.727, 19.0), &contrail);
			SRM1->AddExhaustStream (th_sep[1], _V(-1.1, 0.727, 18.5), &contrail);
			SRM1->AddExhaustStream (th_sep[2], _V(-1.1, 0.727, 18.0), &contrail);
			SRM1->AddExhaustStream (th_sep[3], _V(-1.45, 1.45,-12.75), &contrail);
			SRM1->AddExhaustStream (th_sep[4], _V(-1.45, 1.45,-13.25), &contrail);
			SRM1->AddExhaustStream (th_sep[5], _V(-1.45, 1.45,-13.75), &contrail);
			SRM1->CreateThrusterGroup (th_sep, 6, THGROUP_MAIN);
			SRM1->SetThrusterGroupLevel(THGROUP_MAIN,1.0);

			ph_SRM2  = SRM2->CreatePropellantResource(100,100,1.0); 
			th_sep[0] = SRM2->CreateThruster ( _V( 1.1,-0.727, 19.0), _V(-0.707, 0.707,0), 3500, ph_SRM2, 1e3, 1e3);
			th_sep[1] = SRM2->CreateThruster ( _V( 1.1,-0.727, 18.5), _V(-0.707, 0.707,0), 3500, ph_SRM2, 1e3, 1e3);
			th_sep[2] = SRM2->CreateThruster ( _V( 1.1,-0.727, 18.0), _V(-0.707, 0.707,0), 3500, ph_SRM2, 1e3, 1e3);
			th_sep[3] = SRM2->CreateThruster ( _V( 1.45,-1.45,-12.75), _V(-0.707, 0.707,0), 3500, ph_SRM2, 1e3, 1e3);
			th_sep[4] = SRM2->CreateThruster ( _V( 1.45,-1.45,-13.25), _V(-0.707, 0.707,0), 3500, ph_SRM2, 1e3, 1e3);
			th_sep[5] = SRM2->CreateThruster ( _V( 1.45,-1.45,-13.75), _V(-0.707, 0.707,0), 3500, ph_SRM2, 1e3, 1e3);
			SRM2->AddExhaustStream (th_sep[0], _V( 1.1,-0.727, 19.0), &contrail);
			SRM2->AddExhaustStream (th_sep[1], _V( 1.1,-0.727, 19.5), &contrail);
			SRM2->AddExhaustStream (th_sep[2], _V( 1.1,-0.727, 18.0), &contrail);
			SRM2->AddExhaustStream (th_sep[3], _V( 1.45,-1.45,-12.75), &contrail);
			SRM2->AddExhaustStream (th_sep[4], _V( 1.45,-1.45,-13.25), &contrail);
			SRM2->AddExhaustStream (th_sep[5], _V( 1.45,-1.45,-13.75), &contrail);
			SRM2->CreateThrusterGroup (th_sep, 6, THGROUP_MAIN);
			SRM2->SetThrusterGroupLevel(THGROUP_MAIN,1.0);
		}
		else if (SRM_Type == SRMU) {
			hanSRM1 = oapiCreateVessel(strcat (name1, " SRB 1"), "CVELTitan/SRMUW", vs1);
			hanSRM2 = oapiCreateVessel(strcat (name2, " SRB 2"), "CVELTitan/SRMUE", vs2);
			SRM1 = oapiGetVesselInterface(hanSRM1);
			SRM2 = oapiGetVesselInterface(hanSRM2);
			ph_SRM1  = SRM1->CreatePropellantResource(100,100,1.0); 
			th_sep[0] = SRM1->CreateThruster ( _V(-1.32, 0.0, 14.0), _V( 1,0,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[1] = SRM1->CreateThruster ( _V(-1.32, 0.0, 14.5), _V( 1,0,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[2] = SRM1->CreateThruster ( _V(-1.32, 0.0, 15.0), _V( 1,0,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[3] = SRM1->CreateThruster ( _V(-1.32, 0.0,-14.0), _V( 1,0,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[4] = SRM1->CreateThruster ( _V(-1.32, 0.0,-14.5), _V( 1,0,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[5] = SRM1->CreateThruster ( _V(-1.32, 0.0,-15.0), _V( 1,0,0), 3500, ph_SRM1, 1e3, 1e3);
			SRM1->AddExhaustStream (th_sep[0], _V(-1.32, 0.0, 14.0), &contrail);
			SRM1->AddExhaustStream (th_sep[1], _V(-1.32, 0.0, 14.5), &contrail);
			SRM1->AddExhaustStream (th_sep[2], _V(-1.32, 0.0, 15.0), &contrail);
			SRM1->CreateThrusterGroup (th_sep, 6, THGROUP_MAIN);
			SRM1->SetThrusterGroupLevel(THGROUP_MAIN,1.0);

			ph_SRM2  = SRM2->CreatePropellantResource(100,100,1.0); 
			th_sep[0] = SRM2->CreateThruster ( _V( 1.32, 0.0, 14.0), _V(-1,0,0), 3500, ph_SRM2, 1e3, 1e3);
			th_sep[1] = SRM2->CreateThruster ( _V( 1.32, 0.0, 14.5), _V(-1,0,0), 3500, ph_SRM2, 1e3, 1e3);
			th_sep[2] = SRM2->CreateThruster ( _V( 1.32, 0.0, 15.0), _V(-1,0,0), 3500, ph_SRM2, 1e3, 1e3);
			th_sep[3] = SRM2->CreateThruster ( _V( 1.32, 0.0,-14.0), _V(-1,0,0), 3500, ph_SRM2, 1e3, 1e3);
			th_sep[4] = SRM2->CreateThruster ( _V( 1.32, 0.0,-14.5), _V(-1,0,0), 3500, ph_SRM2, 1e3, 1e3);
			th_sep[5] = SRM2->CreateThruster ( _V( 1.32, 0.0,-15.0), _V(-1,0,0), 3500, ph_SRM2, 1e3, 1e3);
			SRM2->AddExhaustStream (th_sep[0], _V( 1.32, 0.0, 14.0), &contrail);
			SRM2->AddExhaustStream (th_sep[1], _V( 1.32, 0.0, 14.5), &contrail);
			SRM2->AddExhaustStream (th_sep[2], _V( 1.32, 0.0, 15.0), &contrail);
			SRM2->CreateThrusterGroup (th_sep, 6, THGROUP_MAIN);
			SRM2->SetThrusterGroupLevel(THGROUP_MAIN,1.0);	
		}
		else if (SRM_Type == RSRM) {
			hanSRM1 = oapiCreateVessel(strcat (name1, " SRB 1"), "CVELTitan/RSRM", vs1);
			hanSRM2 = oapiCreateVessel(strcat (name2, " SRB 2"), "CVELTitan/RSRM", vs2);
			SRM1 = oapiGetVesselInterface(hanSRM1);
			SRM2 = oapiGetVesselInterface(hanSRM2);
			ph_SRM1  = SRM1->CreatePropellantResource(100,100,1.0); 
			th_sep[0] = SRM1->CreateThruster ( _V(-1.32, 0.0, 20.0), _V( 1,0,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[1] = SRM1->CreateThruster ( _V(-1.32, 0.0, 20.5), _V( 1,0,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[2] = SRM1->CreateThruster ( _V(-1.32, 0.0, 21.0), _V( 1,0,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[3] = SRM1->CreateThruster ( _V(-1.32, 0.0,-18.0), _V( 1,0,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[4] = SRM1->CreateThruster ( _V(-1.32, 0.0,-18.5), _V( 1,0,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[5] = SRM1->CreateThruster ( _V(-1.32, 0.0,-19.0), _V( 1,0,0), 3500, ph_SRM1, 1e3, 1e3);
			SRM1->AddExhaustStream (th_sep[0], _V(-1.32, 0.0, 20.0), &contrail);
			SRM1->AddExhaustStream (th_sep[1], _V(-1.32, 0.0, 20.5), &contrail);
			SRM1->AddExhaustStream (th_sep[2], _V(-1.32, 0.0, 21.0), &contrail);
			SRM1->AddExhaustStream (th_sep[3], _V(-1.45, 0.0,-18.75), &contrail);
			SRM1->AddExhaustStream (th_sep[4], _V(-1.45, 0.0,-18.25), &contrail);
			SRM1->AddExhaustStream (th_sep[5], _V(-1.45, 0.0,-18.75), &contrail);
			SRM1->CreateThrusterGroup (th_sep, 6, THGROUP_MAIN);
			SRM1->SetThrusterGroupLevel(THGROUP_MAIN,1.0);

			ph_SRM2  = SRM2->CreatePropellantResource(100,100,1.0); 
			th_sep[0] = SRM2->CreateThruster ( _V( 1.32, 0.0, 20.0), _V(-1,0,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[1] = SRM2->CreateThruster ( _V( 1.32, 0.0, 20.5), _V(-1,0,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[2] = SRM2->CreateThruster ( _V( 1.32, 0.0, 21.0), _V(-1,0,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[3] = SRM2->CreateThruster ( _V( 1.32, 0.0,-18.0), _V(-1,0,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[4] = SRM2->CreateThruster ( _V( 1.32, 0.0,-18.5), _V(-1,0,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[5] = SRM2->CreateThruster ( _V( 1.32, 0.0,-19.0), _V(-1,0,0), 3500, ph_SRM1, 1e3, 1e3);
			SRM2->AddExhaustStream (th_sep[0], _V( 1.32, 0.0, 20.0), &contrail);
			SRM2->AddExhaustStream (th_sep[1], _V( 1.32, 0.0, 20.5), &contrail);
			SRM2->AddExhaustStream (th_sep[2], _V( 1.32, 0.0, 21.0), &contrail);
			SRM2->AddExhaustStream (th_sep[3], _V( 1.45, 0.0,-18.75), &contrail);
			SRM2->AddExhaustStream (th_sep[4], _V( 1.45, 0.0,-18.25), &contrail);
			SRM2->AddExhaustStream (th_sep[5], _V( 1.45, 0.0,-18.75), &contrail);
			SRM2->CreateThrusterGroup (th_sep, 6, THGROUP_MAIN);
			SRM2->SetThrusterGroupLevel(THGROUP_MAIN,1.0);	
		}
		else {
			if (SRM_Type == UA1205) {
				hanSRM1 = oapiCreateVessel(strcat (name1, " SRB 1"), "CVELTitan/UA1205AW", vs1);
				hanSRM2 = oapiCreateVessel(strcat (name2, " SRB 2"), "CVELTitan/UA1205AE", vs2);
			} else {
				hanSRM1 = oapiCreateVessel(strcat (name1, " SRB 1"), "CVELTitan/UA1205W", vs1);
				hanSRM2 = oapiCreateVessel(strcat (name2, " SRB 2"), "CVELTitan/UA1205E", vs2);
			}
			SRM1 = oapiGetVesselInterface(hanSRM1);
			SRM2 = oapiGetVesselInterface(hanSRM2);
			ph_SRM1  = SRM1->CreatePropellantResource(100,100,1.0); 
			th_sep[0] = SRM1->CreateThruster ( _V(-1.1, 0.727, 12.25), _V( 0.707,-0.707,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[1] = SRM1->CreateThruster ( _V(-1.1, 0.727, 11.75), _V( 0.707,-0.707,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[2] = SRM1->CreateThruster ( _V(-1.1, 0.727, 11.25), _V( 0.707,-0.707,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[3] = SRM1->CreateThruster ( _V(-1.45,0.975,-10.40), _V( 0.707,-0.707,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[4] = SRM1->CreateThruster ( _V(-1.45,0.975,-11.00), _V( 0.707,-0.707,0), 3500, ph_SRM1, 1e3, 1e3);
			th_sep[5] = SRM1->CreateThruster ( _V(-1.45,0.975,-11.50), _V( 0.707,-0.707,0), 3500, ph_SRM1, 1e3, 1e3);
			SRM1->AddExhaustStream (th_sep[0], _V(-1.1, 0.727, 12.25), &contrail);
			SRM1->AddExhaustStream (th_sep[1], _V(-1.1, 0.727, 11.75), &contrail);
			SRM1->AddExhaustStream (th_sep[2], _V(-1.1, 0.727, 11.25), &contrail);
			SRM1->AddExhaustStream (th_sep[3], _V(-1.45,0.975,-10.40), &contrail);
			SRM1->AddExhaustStream (th_sep[4], _V(-1.45,0.975,-11.00), &contrail);
			SRM1->AddExhaustStream (th_sep[5], _V(-1.45,0.975,-11.50), &contrail);
			SRM1->CreateThrusterGroup (th_sep, 6, THGROUP_MAIN);
			SRM1->SetThrusterGroupLevel(THGROUP_MAIN,1.0);

			ph_SRM2  = SRM2->CreatePropellantResource(100,100,1.0); 
			th_sep[0] = SRM2->CreateThruster ( _V( 1.1, -0.727, 12.25), _V(-0.707, 0.707,0), 3500, ph_SRM2, 1e3, 1e3);
			th_sep[1] = SRM2->CreateThruster ( _V( 1.1, -0.727, 11.75), _V(-0.707, 0.707,0), 3500, ph_SRM2, 1e3, 1e3);
			th_sep[2] = SRM2->CreateThruster ( _V( 1.1, -0.727, 11.25), _V(-0.707, 0.707,0), 3500, ph_SRM2, 1e3, 1e3);
			th_sep[3] = SRM2->CreateThruster ( _V( 1.45,-0.975,-10.40), _V(-0.707, 0.707,0), 3500, ph_SRM2, 1e3, 1e3);
			th_sep[4] = SRM2->CreateThruster ( _V( 1.45,-0.975,-11.00), _V(-0.707, 0.707,0), 3500, ph_SRM2, 1e3, 1e3);
			th_sep[5] = SRM2->CreateThruster ( _V( 1.45,-0.975,-11.50), _V(-0.707, 0.707,0), 3500, ph_SRM2, 1e3, 1e3);
			SRM2->AddExhaustStream (th_sep[0], _V( 1.1, -0.727, 12.25), &contrail);
			SRM2->AddExhaustStream (th_sep[1], _V( 1.1, -0.727, 11.75), &contrail);
			SRM2->AddExhaustStream (th_sep[2], _V( 1.1, -0.727, 11.25), &contrail);
			SRM2->AddExhaustStream (th_sep[3], _V( 1.45,-0.975,-10.40), &contrail);
			SRM2->AddExhaustStream (th_sep[4], _V( 1.45,-0.975,-11.00), &contrail);
			SRM2->AddExhaustStream (th_sep[5], _V( 1.45,-0.975,-11.50), &contrail);
			SRM2->CreateThrusterGroup (th_sep, 6, THGROUP_MAIN);
			SRM2->SetThrusterGroupLevel(THGROUP_MAIN,1.0);		
		}
		DelPropellantResource (ph_first);

		DelThrusterGroup (thg_first, THGROUP_MAIN, true);
		DelThrusterGroup (thg_pitchup, THGROUP_ATT_PITCHUP, true);
		DelThrusterGroup (thg_pitchdown, THGROUP_ATT_PITCHDOWN, true);

		DelThrusterGroup (THGROUP_ATT_BANKLEFT, true);
		DelThrusterGroup (THGROUP_ATT_BANKRIGHT, true);
		DelThrusterGroup (THGROUP_ATT_YAWLEFT, true);
		DelThrusterGroup (THGROUP_ATT_YAWRIGHT, true);

		ClearThrusterDefinitions();
		if (fairing > 0)
			SetMainStage ();	
		else SetNoFairStage();
		bsolidsfired = false;
		PlayVesselWave3(MyID,PLAYBANG);
	}		

    else if (stage == 1) 
	{
		vs.vrot.x  = -0.19;
		vs.vrot.y  = 0.0;
		vs.vrot.z  = 0.0;
		vs1.vrot.x = 0.0;	
		vs1.vrot.y = -0.14;	
		vs1.vrot.z = 0.0;	
		vs2.vrot.x = 0.0;	
		vs2.vrot.y = 0.14;	
		vs2.vrot.z = 0.0;
		if (fairing == 101) {
			oapiCreateVessel(strcat (name1, " Titan Fairing L"), "CVELTitan/Fairing101b", vs1);
			oapiCreateVessel(strcat (name2, " Titan Fairing R"), "CVELTitan/Fairing101a", vs2);
		}
		else if (fairing == 102) {
			oapiCreateVessel(strcat (name1, " Titan Fairing L"), "CVELTitan/Fairing102b", vs1);
			oapiCreateVessel(strcat (name2, " Titan Fairing R"), "CVELTitan/Fairing102a", vs2);
		}
		else if (fairing == 201) {
			oapiCreateVessel(strcat (name1, " Titan Fairing L"), "CVELTitan/Fairing201b", vs1);
			oapiCreateVessel(strcat (name2, " Titan Fairing R"), "CVELTitan/Fairing201a", vs2);
		}
		else if (fairing == 300) {
			oapiCreateVessel(strcat (name1, " Titan Fairing L"), "CVELTitan/Fairing300b", vs1);
			oapiCreateVessel(strcat (name2, " Titan Fairing R"), "CVELTitan/Fairing300a", vs2);
		}
		else if (fairing == 301) {
			oapiCreateVessel(strcat (name1, " Titan Fairing L"), "CVELTitan/Fairing301b", vs1);
			oapiCreateVessel(strcat (name2, " Titan Fairing R"), "CVELTitan/Fairing301a", vs2);
		}
		else if (fairing == 311) {
			vs1.vrot.x = 0.07;	
			vs2.vrot.x = 0.07;	
			oapiCreateVessel(strcat (name1, " Titan Fairing L"), "CVELTitan/Fairing311a", vs1);
			oapiCreateVessel(strcat (name2, " Titan Fairing R"), "CVELTitan/Fairing311b", vs2);
			oapiCreateVessel(strcat (name3, " Titan Fairing C"), "CVELTitan/Fairing311c", vs);
		}
		else if (fairing == 312) {
			vs1.vrot.x = 0.07;	
			vs2.vrot.x = 0.07;	
			oapiCreateVessel(strcat (name1, " Titan Fairing L"), "CVELTitan/Fairing312a", vs1);
			oapiCreateVessel(strcat (name2, " Titan Fairing R"), "CVELTitan/Fairing312b", vs2);
			oapiCreateVessel(strcat (name3, " Titan Fairing C"), "CVELTitan/Fairing312c", vs);
		}
		else if (fairing == 313) {
			vs1.vrot.x = 0.07;	
			vs2.vrot.x = 0.07;	
			oapiCreateVessel(strcat (name1, " Titan Fairing L"), "CVELTitan/Fairing313a", vs1);
			oapiCreateVessel(strcat (name2, " Titan Fairing R"), "CVELTitan/Fairing313b", vs2);
			oapiCreateVessel(strcat (name3, " Titan Fairing C"), "CVELTitan/Fairing313c", vs);
		}
		else if (fairing == 314) {
			vs1.vrot.x = 0.07;	
			vs2.vrot.x = 0.07;	
			oapiCreateVessel(strcat (name1, " Titan Fairing L"), "CVELTitan/Fairing314a", vs1);
			oapiCreateVessel(strcat (name2, " Titan Fairing R"), "CVELTitan/Fairing314b", vs2);
			oapiCreateVessel(strcat (name3, " Titan Fairing C"), "CVELTitan/Fairing314c", vs);
		}
		else if (fairing == 401) {
			oapiCreateVessel(strcat (name1, " Titan Fairing L"), "CVELTitan/Fairing401b", vs1);
			oapiCreateVessel(strcat (name2, " Titan Fairing R"), "CVELTitan/Fairing401a", vs2);
		}
		else if (fairing == 402) {
			oapiCreateVessel(strcat (name1, " Titan Fairing L"), "CVELTitan/Fairing402b", vs1);
			oapiCreateVessel(strcat (name2, " Titan Fairing R"), "CVELTitan/Fairing402a", vs2);
		}
		else if (fairing == 500) {
			oapiCreateVessel(strcat (name1, " Titan Fairing L"), "CVELTitan/Fairing500b", vs1);
			oapiCreateVessel(strcat (name2, " Titan Fairing R"), "CVELTitan/Fairing500a", vs2);
		}
		else if (fairing == 501) {
			oapiCreateVessel(strcat (name1, " Titan Fairing L"), "CVELTitan/Fairing501b", vs1);
			oapiCreateVessel(strcat (name2, " Titan Fairing R"), "CVELTitan/Fairing501a", vs2);
		}
		else if (fairing == 601) {
			oapiCreateVessel(strcat (name1, " Titan Fairing L"), "CVELTitan/Fairing601a", vs1);
			oapiCreateVessel(strcat (name2, " Titan Fairing R"), "CVELTitan/Fairing601b", vs2);
		}
		else if (fairing == 611) {
			vs1.vrot.x = 0.07;	
			vs2.vrot.x = 0.07;	
			oapiCreateVessel(strcat (name1, " Titan Fairing L"), "CVELTitan/Fairing611a", vs1);
			oapiCreateVessel(strcat (name2, " Titan Fairing R"), "CVELTitan/Fairing611b", vs2);
			oapiCreateVessel(strcat (name3, " Titan Fairing C"), "CVELTitan/Fairing611c", vs);
		}
		else if (fairing == 612) {
			vs1.vrot.x = 0.07;	
			vs2.vrot.x = 0.07;	
			oapiCreateVessel(strcat (name1, " Titan Fairing L"), "CVELTitan/Fairing612a", vs1);
			oapiCreateVessel(strcat (name2, " Titan Fairing R"), "CVELTitan/Fairing612b", vs2);
			oapiCreateVessel(strcat (name3, " Titan Fairing C"), "CVELTitan/Fairing612c", vs);
		}
		else if (fairing == 613) {
			vs1.vrot.x = 0.07;	
			vs2.vrot.x = 0.07;	
			oapiCreateVessel(strcat (name1, " Titan Fairing L"), "CVELTitan/Fairing613a", vs1);
			oapiCreateVessel(strcat (name2, " Titan Fairing R"), "CVELTitan/Fairing613b", vs2);
			oapiCreateVessel(strcat (name3, " Titan Fairing C"), "CVELTitan/Fairing613c", vs);
		}
		else if (fairing == 614) {
			vs1.vrot.x = 0.07;	
			vs2.vrot.x = 0.07;	
			oapiCreateVessel(strcat (name1, " Titan Fairing L"), "CVELTitan/Fairing614a", vs1);
			oapiCreateVessel(strcat (name2, " Titan Fairing R"), "CVELTitan/Fairing614b", vs2);
			oapiCreateVessel(strcat (name3, " Titan Fairing C"), "CVELTitan/Fairing614c", vs);
		}
		else if (fairing == 615) {
			vs1.vrot.x = 0.07;	
			vs2.vrot.x = 0.07;	
			oapiCreateVessel(strcat (name1, " Titan Fairing L"), "CVELTitan/Fairing615a", vs1);
			oapiCreateVessel(strcat (name2, " Titan Fairing R"), "CVELTitan/Fairing615b", vs2);
			oapiCreateVessel(strcat (name3, " Titan Fairing C"), "CVELTitan/Fairing615c", vs);
		}
		SetNoFairStage ();
		PlayVesselWave3(MyID,PLAYBANG);
  
	}

	else if ((stage == 2) && ((SecndS > 0) || (UprS > 0)))
	{
		vs.vrot.x  = 0.0;
		vs.vrot.y  = 0.0;
		vs.vrot.z  = 0.0;
		
		vs1.vrot.x = 0.0;	
		vs1.vrot.y = 0.0;
		vs1.vrot.z = 0.0;
		
		vs2.vrot.x = 0.0;
		vs2.vrot.y = 0.0;
		vs2.vrot.z = 0.0;
		vs2.fuel = 1.0;

		if ((Titan2) || (Titan1) || (TitanC))
			oapiCreateVessel(strcat (name1, " Titan First Stage"), "CVELTitan/Titan_Stage_First", vs1);
		else if (Titan4)
			oapiCreateVessel(strcat (name1, " Titan First Stage"), "CVELTitan/Titan45_Stage_First", vs1);
		else
			oapiCreateVessel(strcat (name1, " Titan First Stage"), "CVELTitan/Titan34_Stage_First", vs1);

		if (SecndS > 0)
			if (TitanC) oapiCreateVessel(strcat (name2, " Titan Intertage"), "CVELTitan/TitanC_Mid_Section", vs2);
			else if (Titan1) oapiCreateVessel(strcat (name2, " Titan Intertage"), "CVELTitan/Int10_8", vs2);
			else oapiCreateVessel(strcat (name2, " Titan Intertage"), "CVELTitan/Titan_Mid_Section", vs2);

		DelPropellantResource (ph_main);

		DelThrusterGroup (thg_main, THGROUP_MAIN, true);
		DelThrusterGroup (thg_pitchup, THGROUP_ATT_PITCHUP, true);
		DelThrusterGroup (thg_pitchdown, THGROUP_ATT_PITCHDOWN, true);

		DelThrusterGroup (THGROUP_ATT_BANKLEFT, true);
		DelThrusterGroup (THGROUP_ATT_BANKRIGHT, true);
		DelThrusterGroup (THGROUP_ATT_YAWLEFT, true);
		DelThrusterGroup (THGROUP_ATT_YAWRIGHT, true);
		ClearThrusterDefinitions();

		
		if (SecndS > 0) 
			SetSecondStage ();
		
		if (SecndS == 0) {
			bManualSeparate = true;
			status = 3;
			stage = 3;
		}
  		if (!deploy_no_more) PlayVesselWave3(MyID,PLAYBANG);

	}
	else if ((stage == 3) && (UprS > 0))
	{

		vs.vrot.x  = 0.0;
		vs.vrot.y  = 0.0;
		vs.vrot.z  = 0.0;
		
		vs1.vrot.x = 0.0;	
		vs1.vrot.y = 0.0;
		vs1.vrot.z = 0.0;
		
		vs2.vrot.x = 0.0;
		vs2.vrot.y = 0.0;
		vs2.vrot.z = 0.0;
		vs2.fuel = 1.0;

		if (Titan2)
			oapiCreateVessel(strcat (name1, " Titan Second Stage"), "CVELTitan/Titan_Stage_Second", vs1); 
		else if (Titan4)
			oapiCreateVessel(strcat (name1, " Titan Second Stage"), "CVELTitan/Titan45_Stage_Second", vs1); 
		else if (TitanC)
			oapiCreateVessel(strcat (name1, " Titan Second Stage"), "CVELTitan/TitanC_Stage_Second", vs1); 
		else if (Titan1)
			oapiCreateVessel(strcat (name1, " Titan Second Stage"), "CVELTitan/Titan1_Stage_Second", vs1); 
		else
			oapiCreateVessel(strcat (name1, " Titan Second Stage"), "CVELTitan/Titan34_Stage_Second", vs1); 

		if (UprS == 3) oapiCreateVessel(strcat (name2, " Titan Intertage 2"), "TentoFiveFtInterstage", vs2);
			else if (TitanC) oapiCreateVessel(strcat (name2, " Titan Intertage 2"), "CVELTitan/Int13_10", vs2);
				else oapiCreateVessel(strcat (name2, " Titan Intertage 2"), "TenFtInterstage", vs2);
/*
		DelPropellantResource (ph_second);

		DelThrusterGroup (thg_second, THGROUP_MAIN, true);
		DelThrusterGroup (thg_pitchup, THGROUP_ATT_PITCHUP, true);
		DelThrusterGroup (thg_pitchdown, THGROUP_ATT_PITCHDOWN, true);

		DelThrusterGroup (THGROUP_ATT_BANKLEFT, true);
		DelThrusterGroup (THGROUP_ATT_BANKRIGHT, true);
		DelThrusterGroup (THGROUP_ATT_YAWLEFT, true);
		DelThrusterGroup (THGROUP_ATT_YAWRIGHT, true);
		ClearThrusterDefinitions();
*/
		if (!deploy_no_more) PlayVesselWave3(MyID,PLAYBANG);
		SetUpperStage ();


	}
	else {

		if (!deploy_no_more) PlayVesselWave3(MyID,PLAYBANG);
		DeployAPayload();

		/*
		depV = DeployPayload(_V(0,0,0));
		deployed ++;
		if (deployed == shiftfocus)
			hVessel=oapiGetVesselByName(depV->GetName());
			if (hVessel != 0)
				bswitchfocus = true;
		Guidance( false );
		DrawAGemini = 0;

		switch (status) {
			case 0:
				stage=0;
				SetSRMsStage();
				break;
			case 1:
				stage=1;
				SetMainStage();
				break;
			case 2:
				stage=2;
				SetNoFairStage();
				break;
			case 3:
				stage = 3;
				status = 2;
				SetSecondStage();
				break;
			case 4:
				stage = 4;
				status = 4;
				SetUpperStage();
				break;
			}
//		if (UprS > 0)
//			SetUpperStage ();
//		else 
//			SetSecondStage ();
*/
	}


}

void CVELTITAN::JettisonLES()
{
				LEStime = 0.0;
				depV = DeployPayload(_V(0,0,100.0));
				depV->SetThrusterGroupLevel(THGROUP_MAIN,1.0);
				PARTICLESTREAMSPEC contrail = 
				{
			//		0, 0.25, 10.0, 9.0, 0.5, 1.0, 10, 3.0, PARTICLESTREAMSPEC::EMISSIVE,
					0, 0.75, 29, 50.0, 0.5, 0.1, 7, 3.0, PARTICLESTREAMSPEC::EMISSIVE,
					PARTICLESTREAMSPEC::LVL_SQRT, 0.1, 1,
					PARTICLESTREAMSPEC::ATM_PLOG, 0, 0
				};

				ph_SRM1  = depV->CreatePropellantResource(1,1,1.0); 
				th_sep[0] = depV->CreateThruster ( _V(0,0,0), _V( 0,0,1), 0.001, ph_SRM1, 1e3, 1e3);
				depV->AddExhaustStream (th_sep[0], _V(0,0,0), &contrail);
				depV->SetThrusterLevel(th_sep[0],1.0);
				ClearMeshesEx();
				PlaceComponents();
				Draw();
			
				CalcWeights();
}

void CVELTITAN::DeployAPayload()
{
				if (LastDeployed >= (oldsimt-dt)) return;				// Ensures jettison requests don't bunch up too close together
				LastDeployed = oldsimt+dt;								// ...a last-chance catch for progrmaming sloppiness
				DrawAGemini = 0;
				if (atchmass > 0.0) {
					DetachChild (attachpos, 2.0);
					atchmass = 0.0;
					depV = AtchV;
					CalcWeights();
				} else {
					depV = DeployPayload(_V(0,0,0));
				}
				if (bAborting)
					depV->SetThrusterGroupLevel(THGROUP_MAIN, 1.0);
				deployed++;
				if (Guidance()) {
					SendCentralMessage(depV, "ON", "AUTOPILOT");
					Guidance(false);
				}
				if (deployed > shiftfocus) deploy_no_more = true;
				if (deployed == shiftfocus) {
					hVessel=oapiGetVesselByName(depV->GetName());
					if (hVessel != 0)
						bswitchfocus = true;
//						oapiSetFocusObject(hVessel);
					shiftfocus = 0;
					deploy_no_more = true;
				}
				ClearMeshesEx();
				PlaceComponents();
				Draw();

				switch (status) {
				case 0:
					stage=0;
					if (Cluster) {
						double thrustlvl = GetThrusterGroupLevel(THGROUP_MAIN);
						SetClusterStage();
						SetThrusterGroupLevel(THGROUP_MAIN,thrustlvl);
					} else {
						bool solidsfiring;
						if (GetThrusterGroupLevel(THGROUP_MAIN) > 0.0)
							solidsfiring = true;
						SetSRMsStage();
						if (solidsfiring)
							SetThrusterGroupLevel(THGROUP_MAIN, 1.0);
					}
					break;
				case 1:
					stage=1;
					SetMainStage();
					SetThrusterGroupLevel(THGROUP_MAIN, 0.0);
					break;
				case 2:
					stage=2;
					SetNoFairStage();
					SetThrusterGroupLevel(THGROUP_MAIN, 0.0);
					break;
				case 3:
					stage = 3;
					status = 3;
					SetSecondStage();
					SetThrusterGroupLevel(THGROUP_MAIN, 0.0);
					break;
				case 4:
					stage = 4;
					status = 4;
					SetUpperStage();
					SetThrusterGroupLevel(THGROUP_MAIN, 0.0);
					break;
				}
				Guidance( false );

}

void CVELTITAN::SetAStage()
{
				switch (status) {
				case 0:
					SetSRMsStage();
					break;
				case 1:
					SetMainStage();
					break;
				case 2:
					SetNoFairStage();
					break;
				case 3:
					SetSecondStage();
					break;
				case 4:
					SetUpperStage();
					break;
				case 13:
					SetSecondStage();
					break;
				}
}

int CVELTITAN::ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate)
{
		if (KEYDOWN (keystate, OAPI_KEY_J) && !KEYMOD_CONTROL(keystate) && !KEYMOD_SHIFT(keystate) ) 
		{ // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
			{
				if (deployed > 0) {
					DeployAPayload();
					return 1;
				}
				if (numSRMs1 > 0) return 1;
				if ((!bsolidsfired) || (bThrustTerm))
					bManualSeparate = true;
				deploy_no_more = false;
				return 1;
			}
		}
		if (KEYDOWN (keystate, OAPI_KEY_U) && !KEYMOD_CONTROL(keystate) && !KEYMOD_SHIFT(keystate) ) 
		{ // "Set launch Azimuth"
			if (oapiAcceptDelayedKey (OAPI_KEY_U, 1.0))
			{
				GetInclinationSP();
				return 1;
			}
		}
		if (KEYDOWN (keystate, OAPI_KEY_E) && KEYMOD_CONTROL(keystate) && !KEYMOD_SHIFT(keystate) ) 
		{ // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0))
			{
				OverridePayloadVelocity(_V(0,0,+17.0));
				PlayVesselWave3(MyID,PLAYABORT);
				Guidance(false);
				if (bThrustTerm) {
					bAborting = true;
					bManualSeparate = true;
				}
				DeployAPayload();

/*
				DrawAGemini = 0;
				depV = DeployPayload(_V(0,0,0));
				hVessel=oapiGetVesselByName(depV->GetName());
				if (hVessel != 0)
					bswitchfocus = true;
//					oapiSetFocusObject(hVessel);
				switch (status) {
				case 0:
					stage=0;
					bool solidsfiring;
					if (GetThrusterGroupLevel(THGROUP_MAIN) > 0.0)
						solidsfiring = true;
					SetSRMsStage();
					if (solidsfiring)
						SetThrusterGroupLevel(THGROUP_MAIN, 1.0);
					break;
				case 1:
					stage=1;
					SetMainStage();
					SetThrusterGroupLevel(THGROUP_MAIN, 0.0);
					break;
				case 2:
					stage=2;
					SetNoFairStage();
					SetThrusterGroupLevel(THGROUP_MAIN, 0.0);
					break;
				case 3:
					stage = 3;
					status = 2;
					SetSecondStage();
					SetThrusterGroupLevel(THGROUP_MAIN, 0.0);
					break;
				case 4:
					stage = 4;
					status = 4;
					SetUpperStage();
					SetThrusterGroupLevel(THGROUP_MAIN, 0.0);
					break;
				}

*/
//				deployed ++;
//				Guidance( false );

				return 1;
			}
		}
		else if (KEYDOWN (keystate, OAPI_KEY_O) && !KEYMOD_CONTROL(keystate) && !KEYMOD_SHIFT(keystate))
		{ 
			if (oapiAcceptDelayedKey (OAPI_KEY_O, 1.0))
				if (!guidance) { 
					Guidance( true );
//					guidance=true;
					sprintf(oapiDebugString(),"Guidance ON");
				}
				else {
					Guidance( false );
//					guidance = false;
					SetAttitudeRotLevel(_V(0,0,0));
					pitchinput = 0.0;
					yawinput = 0.0;
					rollinput = 0.0;

					sprintf(oapiDebugString(),"Guidance OFF");
				}
			return 1;
		}
		
		return 0;
}

void CVELTITAN::SetClassCaps(  FILEHANDLE cfg )
{
	ph_SRMs0          = NULL;
	ph_SRMs1          = NULL;
	ph_first          = NULL;
	ph_second         = NULL;
	ph_main			 = NULL;
	thg_first        = NULL;
	thg_second       = NULL;
	thg_main		 = NULL;
	thg_upper		 = NULL;
	thg_pitchup		 = NULL;
	thg_pitchdown	 = NULL;

	// if we have orbitersound we load up some custom sounds
/*	if(!ConnectToOrbiterSoundDLL())
		havesound=false;
	else
	{
		havesound=true;
	}
*/
//	deployed = 0;
	if (status == 2) SetNoFairStage();
	else SetSecondStage();

	CheckCentralMessage();   


}

void CVELTITAN::PlaceComponents()
{
	COMMON_MESH tmp;
	VECTOR3 MeshPos;

	if (numSRMs0 > 0) {
		tmp.MeshHandle(hSRM4);
		MeshPos = SRM4_OFS+OFFSIDEL;
		MeshPos.z += Stg0Longer + Stg1Longer + Stg2Longer;
		tmp.MeshPosition(MeshPos);
		AddMeshEx (tmp);
		tmp.MeshHandle(hSRM5);
		MeshPos = SRM5_OFS+OFFSIDER;
		MeshPos.z += Stg0Longer + Stg1Longer + Stg2Longer;
		tmp.MeshPosition(MeshPos);
		AddMeshEx (tmp);
		tmp.MeshHandle(hSRM6);
		MeshPos = SRM6_OFS+OFFSIDEL;
		MeshPos.z += Stg0Longer + Stg1Longer + Stg2Longer;
		tmp.MeshPosition(MeshPos);
		AddMeshEx (tmp);
		tmp.MeshHandle(hSRM7);
		MeshPos = SRM7_OFS+OFFSIDER;
		MeshPos.z += Stg0Longer + Stg1Longer + Stg2Longer;
		tmp.MeshPosition(MeshPos);
		AddMeshEx (tmp);
		tmp.MeshHandle(hSRM8);
		MeshPos = SRM8_OFS+OFFSIDEL;
		MeshPos.z += Stg0Longer + Stg1Longer + Stg2Longer;
		tmp.MeshPosition(MeshPos);
		AddMeshEx (tmp);
		tmp.MeshHandle(hSRM9);
		MeshPos = SRM9_OFS+OFFSIDER;
		MeshPos.z += Stg0Longer + Stg1Longer + Stg2Longer;
		tmp.MeshPosition(MeshPos);
		AddMeshEx (tmp);
	}

	if (numSRMs1 > 0) {
		tmp.MeshHandle(hSRM0);
		MeshPos = SRM0_OFS;
		MeshPos.z += Stg1Longer + Stg2Longer;
		tmp.MeshPosition(MeshPos);
		AddMeshEx (tmp);
		tmp.MeshHandle(hSRM1);
		MeshPos = SRM1_OFS;
		MeshPos.z += Stg1Longer + Stg2Longer;
		tmp.MeshPosition(MeshPos);
		AddMeshEx (tmp);
		if (numSRMs1 == 4) {
			tmp.MeshHandle(hSRM2);
			MeshPos = SRM2_OFS;
			MeshPos.z += Stg1Longer + Stg2Longer;
			tmp.MeshPosition(MeshPos);
			AddMeshEx (tmp);
			tmp.MeshHandle(hSRM3);
			MeshPos = SRM3_OFS;
			MeshPos.z += Stg1Longer + Stg2Longer;
			tmp.MeshPosition(MeshPos);
			AddMeshEx (tmp);
		}
	}

	if (status < 1) {
		if (Cluster) {
			tmp.MeshHandle(hCluster);
			MeshPos = stage0_ofs_left;
			MeshPos.z += Stg1Longer + Stg2Longer;
			tmp.MeshPosition(MeshPos);
			AddMeshEx (tmp);
			tmp.MeshHandle(hClusterInt);
			MeshPos = stage0_ofs_left;
			MeshPos.z += Stg1Longer + Stg2Longer;
			tmp.MeshPosition(MeshPos);
			AddMeshEx (tmp);
		} else {
			tmp.MeshHandle(hsrm2);
			MeshPos = stage0_ofs_left;
			MeshPos.z += Stg1Longer + Stg2Longer;
			tmp.MeshPosition(MeshPos);
			AddMeshEx (tmp);
			tmp.MeshHandle(hsrm1);
			MeshPos = stage0_ofs_right;
			MeshPos.z += Stg1Longer + Stg2Longer;
			tmp.MeshPosition(MeshPos);
			AddMeshEx (tmp);
		}
	}
	if (status < 3) {
		tmp.MeshHandle(hFirstStage);
		MeshPos = STAGE1_OFS;
		MeshPos.z += Stg1Longer + Stg2Longer + Stg1Bias;
		if (invoked) MeshPos = _V(0,0,0);
		tmp.MeshPosition(MeshPos);
		AddMeshEx (tmp);
		if (SecndS > 0) {
			tmp.MeshHandle(hInterStage);
			MeshPos = MID_OFS;
			MeshPos.z += Stg2Longer;
			tmp.MeshPosition(MeshPos);
			AddMeshEx (tmp);
		}
	}
	if ((status < 2) && (fairing != 0))  {
		tmp.MeshHandle(hshroud2);
		tmp.MeshPosition(ofs_shroud2);
		AddMeshEx (tmp);
		tmp.MeshHandle(hshroud1);
		tmp.MeshPosition(ofs_shroud1);
		AddMeshEx (tmp);
		tmp.MeshHandle(hshroud3);
		tmp.MeshPosition(ofs_shroud3);
		AddMeshEx (tmp);
	}
	if (((status < 4) && (SecndS > 0)) || ((deploy_no_more) && (UprS == 0))) {
		tmp.MeshHandle(hSecondStage);
		MeshPos = STAGE2_OFS;
		MeshPos.z += Stg2Bias + Stg2Longer;
		if (invoked) MeshPos = _V(0,0,0);
		tmp.MeshPosition(MeshPos);
//		if (invoked) tmp.MeshPosition(STAGE2_OFS + _V(0,0,-10));
		if (SecndS > 0)
			AddMeshEx (tmp);
	}
	if ((status < 5) && (UprS == 1)) {
		tmp.MeshHandle(hTranstage);
		tmp.MeshPosition(TRANSTAGE_OFS);
		AddMeshEx (tmp);
	}
	if ((status < 5) && (UprS == 2)) {
		tmp.MeshHandle(hCentaur);
		tmp.MeshPosition(CENTAUR_OFS);
		AddMeshEx (tmp);
	}
	if ((status < 5) && (UprS == 3)) {
		tmp.MeshHandle(hAgena);
		tmp.MeshPosition(AGENA_OFS);
		AddMeshEx (tmp);
	}
	if ((status < 5) && (UprS == 4)) {
		tmp.MeshHandle(hChariot);
		tmp.MeshPosition(CHARIOT_OFS);
		AddMeshEx (tmp);
	}
	if ((status < 4) && (UprS > 0)) {
			if (UprS == 3) tmp.MeshHandle(hInterStage2b);
				else if (TitanC) tmp.MeshHandle(hInterStage2c);
					else tmp.MeshHandle(hInterStage2);
			tmp.MeshPosition(INTERSTAGE2_OFS);
			AddMeshEx (tmp);
		}
	if (DrawAGemini > 0) {
		tmp.MeshHandle(hGemCapsule);
		tmp.MeshPosition(OFS_GEM_CAPSULE+_V(0,0,GeminiZ));
		AddMeshEx(tmp);
		tmp.MeshHandle(hGemHeatshield);
		tmp.MeshPosition(OFS_GEM_HEATSHIELD+_V(0,0,GeminiZ));
		AddMeshEx(tmp);
		tmp.MeshHandle(hGemCdrSeat);
		tmp.MeshPosition(OFS_GEM_CDR+_V(0,0,GeminiZ));
		AddMeshEx(tmp);
		tmp.MeshHandle(hGemPltSeat);
		tmp.MeshPosition(OFS_SEAT+_V(0,0,GeminiZ));
		AddMeshEx(tmp);
		tmp.MeshHandle(hGemDoorL);
		tmp.MeshPosition(OFS_GEM_DOOR_LEFT+_V(0,0,GeminiZ));
		AddMeshEx(tmp);
		tmp.MeshHandle(hGemDoorR);
		tmp.MeshPosition(OFS_GEM_DOOR_RIGHT+_V(0,0,GeminiZ));
		AddMeshEx(tmp);
		tmp.MeshHandle(hGemNose);
		tmp.MeshPosition(OFS_GEM_NOSE+_V(0,0,GeminiZ));
		AddMeshEx(tmp);
	}
	if (DrawAGemini > 1) {
		tmp.MeshHandle(hGemEquip);
		tmp.MeshPosition(OFS_GEM_EQUIPMENT+_V(0,0,GeminiZ));
		AddMeshEx(tmp);
		tmp.MeshHandle(hGemAdapt);
		tmp.MeshPosition(OFS_GEM_ADAPTER+_V(0,0,GeminiZ));
		AddMeshEx(tmp);
	}
}

void CVELTITAN::TimeStep(double simt )
{



	ZeroMemory (key,256);
	ZeroMemory (message,256);

	if( CheckCentralMessage(key, message) ) {
		CheckCentralMessage();
		if (( strstr(key, "TOGGLE" ) ) && ( strstr(message, "AUTOPILOT" ) )) {
			if (!guidance) { 
				Guidance( true );
			}
			else {
				Guidance( false );
				SetAttitudeRotLevel(_V(0,0,0));
				pitchinput = 0.0;
				yawinput = 0.0;
				rollinput = 0.0;
			}
		}
		else if (( strstr(key, "INCSET" ) ) && ( strstr(message, "AUTOPILOT" ) ))
				GetInclinationSP();
		else if (( strstr(key, "ON" ) ) && ( strstr(message, "THROTTLE" ) ))
				SetThrusterGroupLevel(THGROUP_MAIN,1.0);
		else if (( strstr(key, "ON" ) ) && ( strstr(message, "JETTISON" ) )) {
				if (deployed > 0) {
					DeployAPayload();
				}
				else if (numSRMs1 == 0) {
				if ((!bsolidsfired) || (bThrustTerm))
					bManualSeparate = true;
				deploy_no_more = false;
				}
		}
		else if (( strstr(key, "ON" ) ) && ( strstr(message, "ABORT" ) )) {
				OverridePayloadVelocity(_V(0,0,+17.0));
				PlayVesselWave3(MyID,PLAYABORT);
				Guidance(false);
				if (bThrustTerm) {
					bAborting = true;
					bManualSeparate = true;
				}
				DeployAPayload();
		}

		while( CheckCentralMessage() ) ;
			SetAStage();
	}


//	if( CheckCentralMessage() ) {
//		while( CheckCentralMessage() ) ;
//	}


	dt = simt - oldsimt;
	oldsimt = simt;

	if (LEStime > 1.0)
		if (((MET - dt) < LEStime) && (MET > LEStime))
			JettisonLES();

	if ((displaytime > 0) && (oapiGetFocusObject() == GetHandle())) displaytime += dt;

	if ((!guidance) && (displaytime < 5) && (oapiGetFocusObject() == GetHandle()))
	{
		GetEquPos( Longitude, Latitude, Radius );
		sprintf(oapiDebugString(), "Target inclination (Equatorial): %.3f", (180 - acos(sin(d_heading*PI/180)*cos(Latitude))*DEG));
	}

	if ((!guidance) && (fabs(oapiGetSimMJD()-LaunchMJD) < .00013)) {
		Guidance(true);
	}

	if ((status == 1) || (status == 2))					// Throttle back to maintain 6.0 G's (really only for SSTO case)
		if (((GetThrusterGroupLevel(THGROUP_MAIN)*thrust_main_vac*2)/GetMass()) > 60.0)
			SetThrusterGroupLevel(THGROUP_MAIN,(GetThrusterGroupLevel(THGROUP_MAIN)*0.98));

	if ((numSRMs1 > 0) || (numSRMs0 > 0))
		if (GetThrusterGroupLevel(THGROUP_MAIN) > 0.0)
			SetThrusterGroupLevel (thg_SRMs,1.0);


	if (numSRMs0 > 0)
		if (GetPropellantMass(ph_SRMs0) == 0.0)
			StageSRMs();
		
	if (numSRMs1 > 0)
		if (GetPropellantMass(ph_SRMs1) == 0.0)
			StageSRMs();

	if ((status < 1) && (!Cluster))
		if ((MET > 0.0) || (!guidance))
			if ((GetThrusterGroupLevel(THGROUP_MAIN) > 0.0) || MET > 0.0 || (bsolidsfired)) {
//				SetThrusterGroupLevel(THGROUP_MAIN,1.0);
				double remaining_fuel =  GetFuelMass()/GetMaxFuelMass();
				double thrustlevel = (0.65+(remaining_fuel*remaining_fuel));
				SetThrusterGroupLevel(THGROUP_MAIN, thrustlevel);
				bsolidsfired = true;
//				SetThrusterMax0(th_srb[0],thrustlevel);
//				SetThrusterMax0(th_srb[1],thrustlevel);
			}

	if ((status < 1) && (Cluster) && (GetThrusterGroupLevel(THGROUP_MAIN) > 0.0))
		if ((MET > 0.0) || (!guidance))
			if ((!ClusterDecreased) && ((GetThrusterMax0(th_srb[0])*8)/GetMass()) > 6.0*G) {
				ClusterDecreased = true;
				double thlvl = GetThrusterGroupLevel(THGROUP_MAIN);
				DelThruster(th_srb[4]);
				DelThruster(th_srb[5]);
				DelThruster(th_srb[6]);
				DelThruster(th_srb[7]);
//				DelThrusterGroup(THGROUP_MAIN,false);
//				thg_main = CreateThrusterGroup (th_srb, 4, THGROUP_MAIN);
				SetThrusterGroupLevel(THGROUP_MAIN,thlvl);
		}

	if (!guidance && !MECO_Cond ) {
		if(!NewIncSP) {
			// Set the Inclination setpoint for the guidance computer
			GetEquPos( Longitude, Latitude, Radius );
// Originally 258.40 to 133.44, changed to allow 270 launch hdg.
// This is BAD for error-checking
//			if (PadHeading < 358.40 && PadHeading > 133.44) {
//				Clairaut_Constant = sin((PadHeading - 180)*RAD) * cos(Latitude);
				INCLINATION = acos(Clairaut_Constant)*DEG;
//			} else {
//				BadHeading = true ;
//			}
			
		}
	}	

	//Check if guidance is on and if so, call
	if (guidance)
	{

		if (BadHeading) 
		{
			sprintf(oapiDebugString(),"Guidance system cannot launch from this heading.");
			guidance = false;
		} 
		
		
		else 
		{
			
			
			// Calculate MET
			if (TZero == -100) 
			{
				MET = -5 ;
				TZero = 0;
			}

			
			else 
			{
				MET = simt - TZero;
			}

			// Call the guidance routine
			Guidance_Routine(this, simt, SimT_Last);
			SimT_Last = MET;
		
		}
	}


	VectorThrust();
/*	if (deployed > 0) deployed++;
	if (deployed > 100) {
		deployed = 0;	
		oapiSetFocusObject (depV->GetHandle());
	}
*/
//	if (deployed == 1) {
//		OBJHANDLE hDeployed;
//		hDeployed = depV->GetHandle();
//		oapiSetFocusObject(hDeployed);
//	}
		
	// Display the inclination setpoint if the switch is on.
	if (DisplayIncSP) {
		sprintf(oapiDebugString(),"Inclination Setpoint: %.2f deg", INCLINATION);
	}
	

	if (bswitchfocus) {
		bswitchfocus = false;
		if (oapiGetVesselByName(GetName()) == oapiGetFocusObject())
			oapiSetFocusObject(hVessel);
	}

	if (stage == -1)
	{
    	if (GetEngineLevel(ENGINE_MAIN) > .95 && !engineignited)
		{
			ignition_time = simt;
			engineignited = true;
		}

		last_simt = simt;            

		if (GetPropellantMass(ph_GEMs) == 0 || bManualSeparate)
		{
			time=simt;
			SeparateStage (stage);
			bManualSeparate=false;
			stage = 0;
		} 

	}	
	if (stage == 0)
	{
    	if (GetEngineLevel(ENGINE_MAIN) > .95 && !engineignited)
		{
			ignition_time = simt;
			engineignited = true;
		}

		last_simt = simt;            

		if (GetPropellantMass(ph_first) == 0 || bManualSeparate)
		{
			time=simt;
			SeparateStage (stage);
			bManualSeparate=false;
			stage = 1;
		} 

	}
	
	else if (stage == 1)
	{
		last_simt = simt;
		
		if (GetAtmPressure() < 10 || GetPropellantMass(ph_main) == 0 || bManualSeparate)
		{
			time=simt;
			bManualSeparate=false;
			SeparateStage (stage);
			stage = 2;
		} 
		
	}
	else if (stage == 2)
	{
		last_simt = simt;
		
		if (GetPropellantMass(ph_main) == 0 ||bManualSeparate)
		{
			stage_sep = simt;
			bManualSeparate=false;
			SeparateStage (stage);
			if (SecndS > 0) stage = 3;
		}
	}
    else if (stage == 3)
	{
		last_simt = simt;

		if (GetPropellantMass(ph_second) == 0 || bManualSeparate)
		{
			if (!deploy_no_more) {
				stage_sep = simt;
				bManualSeparate=false;
				SeparateStage (stage);
				if (UprS > 0) stage = 4;
			}
		}
	}
    else if (stage == 4)
	{
		last_simt = simt;

		if (GetPropellantMass(ph_upper) == 0 || bManualSeparate)
		{
			stage_sep = simt;
			bManualSeparate=false;
			SeparateStage (stage);
			stage = 5;
		}
	}

	if ((status == 3) && (GetPropellantMass(ph_second) == 0))
		status = 13;

	if (bAborting)
		SetThrusterGroupLevel(THGROUP_MAIN, 0.0);


	if (Firstsweep) {
		Firstsweep = false;
		if (mode > 0) {
			hV = oapiGetVesselByName(tgtname);
			if (hV != 0) {
				AtchV = oapiGetVesselInterface (hV);
				atchmass = AtchV->GetMass();
		
					//	  		       			       child   pos           dir           rot      handle
					tgtatch = AtchV->CreateAttachment (true, tgtpos,   _V(0,0,-1),       tgtrot, "X");

					if (AtchV->GetAttachmentStatus(attachpos) == NULL) 
						AttachChild (hV, attachpos, tgtatch);

					CalcWeights();
//				SetMainStage();

			}
		}
		ProgramAuto();
	}

	if (mode > 0)
		if (AtchV->GetThrusterGroupLevel(THGROUP_MAIN) > 0.0)
			Guidance( true );
}




void CVELTITAN::SaveState( FILEHANDLE scn )
{
	char cbuf[256];
	SaveDefaultState (scn);

	oapiWriteScenario_int (scn, "CONFIGURATION", status);
	oapiWriteScenario_float (scn, "TIME", last_simt-ignition_time);
	oapiWriteScenario_int (scn, "SRM_TYPE", SRM_Type);
	oapiWriteScenario_float (scn, "SECOND STAGE", SecndS);
	oapiWriteScenario_int (scn, "UPPER STAGE", UprS);
	oapiWriteScenario_float (scn, "AUTO LEO", autoLEO);
	oapiWriteScenario_int (scn, "THRUST_TERM", bThrustTerm);
	oapiWriteScenario_float (scn, "TGT_HEADING", PadHeading - 180.0);
	oapiWriteScenario_float (scn, "PADBIAS", padbias);
	oapiWriteScenario_int (scn, "SHIFTFOCUS", shiftfocus);
	oapiWriteScenario_int (scn, "STG2_RCS", SecondStageRCS);
	oapiWriteScenario_int (scn, "NUM_SRMS", numSRMs1+numSRMs0);
	oapiWriteScenario_int (scn, "FAIRING", fairing);
	oapiWriteScenario_float (scn, "FAIRLOC", ofs_shroud1.z);
	oapiWriteScenario_vec (scn, "CAMERA", CAMERA_OFS);
	if (DrawAGemini > 0) {
		oapiWriteScenario_int (scn, "DRAWAGEMINI",DrawAGemini);
		oapiWriteScenario_float (scn, "GEMINIZ",GeminiZ);
	}
	oapiWriteScenario_float (scn, "LAUNCH MJD", LaunchMJD);
	sprintf (cbuf, "%s %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f", tgtname, tgtpos.x, tgtpos.y, tgtpos.z, tgtrot.x, tgtrot.y, tgtrot.z);
	if (atchmass > 0.0)
		oapiWriteScenario_string (scn, "PAYATCH", cbuf);
	if (LEStime > 0)
		oapiWriteScenario_float (scn, "LES", LEStime);


	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Call this after the normal SaveDefautState() to record attached payloads
	/* it the scenario file */

	/* save payload lines */
	SaveDefaultStateEx(scn);
}


void CVELTITAN::VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount ) 
{
//	setstatus(status);
//	SetUpOrion();
	ClearMeshesEx();
	PlaceComponents();
	Draw();
}

void CVELTITAN::ProgramAuto( )
{

	if (UprS == 1)
		Uppermass = TRANSTAGE_PROP_MASS + TRANSTAGE_DRYMASS;
	else if (UprS == 2)
		Uppermass = CENTAUR_PROP_MASS + CENTAUR_DRYMASS;
	else if (UprS == 3)
		Uppermass = AGENA_PROP_MASS + AGENA_DRYMASS;
	else if (UprS == 4)
		Uppermass = CHARIOT_PROP_MASS + CHARIOT_DRYMASS;

	timebias = ((atchmass + CalculateMass()+Uppermass)-13000.0)/270.0;
	if (status > 0) timebias += (10000+Uppermass)/300.0;
	if (((TitanC) || (Titan2) || (Titan1)) && (status > 0)) timebias += 60.0 - 4*(numSRMs1+numSRMs0);
	if (timebias < 0) timebias = 0.0;

	double stagebias = 1.0;
	double stagebias15 = 1.0;
	double stagebias2 = 1.0;
	if ((status < 2) && (status > 0) && (SecndS == 0))	 {		// special autopilot kludge for SSTO case
//		stagebias = 0.25;
//		stagebias15 = stagebias * 1.5;
//		stagebias2 = stagebias * 2;
//		stagebias3 = stagebias * 3;
		timebias += -5.0;
		stagebias = 0.75;
//		turnbias = -100.0;
		SSTOcase = true;
	}
	if (Cluster) {
		SSTOcase = false;
		timebias += -35.0;
		stagebias = 0.65;
	}

	N=14;
	double *fmet = new double[N];
	double *fcpitch = new double[N];

	fmet[0]					=    0.0;
	fmet[1]					=   20.0;
	fmet[2]					=   30.0;
	fmet[3]					=  (37.0+timebias/2);
	fmet[4]					=  (45.0+timebias/1.5);
	fmet[5]					=  (53.0+timebias);
	fmet[6]					=  (60.0+timebias)*(stagebias2);
	fmet[7]					=  (68.0+timebias*1.25)*(stagebias15);
	fmet[8]					= (165.0+timebias*1.5)*(stagebias);
	fmet[9]					= (240.0+timebias*1.5)*stagebias;
	fmet[10]				= (290.0*1.5+timebias*2)*stagebias;
	fmet[11]				= (310.0+timebias*2)*stagebias;
	fmet[12]				= (360.0+timebias*2.25)*stagebias;
	fmet[13]				= (400.0+timebias*2.5)*stagebias;

	fcpitch[0]				= 89.0;
	fcpitch[1]				= 85.0;
	fcpitch[2]				= 80.0;
	fcpitch[3]				= 70.0;
	fcpitch[4]				= 60.0;
	fcpitch[5]				= 50.0;
	fcpitch[6]				= 40.0;
	fcpitch[7]				= 30.0;
	fcpitch[8]				= 20.0;
	fcpitch[9]				= 15.0;
	fcpitch[10]				= 10.0;
	fcpitch[11]				=  5.0;
	fcpitch[12]				=  2.5;
	fcpitch[13]				=  0.0;
	ProgramPitchMet( N, fmet, fcpitch );



}

void CVELTITAN::LoadState(  FILEHANDLE scn, void *vs )
{
    char *line;
//	double dummy;
	double temp;

	while (oapiReadScenario_nextline (scn, line)) 
	{

		/* CVE-lite function ! */
		/* IMPORTANT */
		/* Call this for every line in the scenario file. It will automatically
		/* deduce which lines are for it, and load the payloads from the file. */

		/* load payload lines */

		ParsePayloadString(line);


		ParseScenarioLineEx (line, vs);

	

        if (!strnicmp (line, "CONFIGURATION", 13)) {
            sscanf (line+13, "%d", &status);
		} else if (!strnicmp (line, "TIME", 4)) {
            sscanf (line+4, "%lf", &temp);
			ignition_time=-temp;
		} else if (!strnicmp (line, "SRM_TYPE", 8)) {
            sscanf (line+8, "%d", &SRM_Type);
		} else if (!strnicmp (line, "SECOND STAGE", 12)) {
            sscanf (line+12, "%d", &SecndS);
		} else if (!strnicmp (line, "UPPER STAGE", 11)) {
            sscanf (line+11, "%d", &UprS);
		} else if (!strnicmp (line, "AUTO LEO", 8)) {
            sscanf (line+8, "%lf", &temp);
			if (temp == 0) autoLEO = false;
			else autoLEO = true;
		} else if( !strncmp( line, "TGT_HEADING", 11 ) ) {
            sscanf (line+11, "%lf", &d_heading);
			PadHeading = d_heading + 180.0;  // It's not pad heading any more; just target heading
			if (PadHeading > 450.0) PadHeading -= 360.0;
			INCLINATION = acos(sin(PadHeading*PI/180)*cos(Latitude))*DEG;
			d_heading = PadHeading;
		} else if (!strnicmp (line, "HEADING", 7)) {
			sscanf(line+7, "%lf", &initialheading);
		} else if (!strnicmp (line, "PADBIAS", 7)) {
			sscanf(line+7, "%lf", &padbias);
		} else if (!strnicmp (line, "SHIFTFOCUS", 10)) {
			sscanf(line+10, "%d", &shiftfocus);
		} else if (!strnicmp (line, "FAIRING", 7)) {
			sscanf(line+7, "%d", &fairing);		
		} else if (!strnicmp (line, "FAIRLOC", 7)) {
			sscanf(line+7, "%lf", &ofs_shroud1.z);
			ofs_shroud2.z = ofs_shroud1.z;  
			ofs_shroud3.z = ofs_shroud1.z;  
		} else if (!strnicmp (line, "DRAWAGEMINI", 11)) {
			sscanf(line+11, "%d", &DrawAGemini);		
		} else if (!strnicmp (line, "GEMINIZ", 7)) {
			sscanf(line+7, "%lf", &GeminiZ);
		} else if (!strnicmp (line, "LAUNCH MJD", 10)) {
			sscanf(line+10, "%lf", &LaunchMJD);	
		} else if (!strnicmp (line, "THRUST_TERM", 11)) {
			sscanf(line+11, "%d", &bThrustTerm);
		} else if (!strnicmp (line, "STG2_RCS", 8)) {
			sscanf(line+8, "%d", &SecondStageRCS);	
		} else if (!strnicmp (line, "NUM_SRMS", 8)) {
			sscanf(line+8, "%d", &numSRMs1);
		} else if (!strnicmp (line, "PAYATCH", 7)) {
			sscanf (line+7, "%31s%lf%lf%lf%lf%lf%lf", tgtname, &tgtpos.x,&tgtpos.y,&tgtpos.z,&tgtrot.x,&tgtrot.y,&tgtrot.z);
			mode = 1;
		} else if (!strnicmp (line, "CAMERA", 6)) {
			sscanf(line+6, "%lf%lf%lf", &CAMERA_OFS.x, &CAMERA_OFS.y, &CAMERA_OFS.z);
		} else if (!strnicmp (line, "LES", 3)) {
			sscanf(line+3, "%lf", &LEStime);
		}
    }
	SetBoosters();

	invoked = false;
	if (DrawAGemini) {
		hGemEquip = oapiLoadMeshGlobal("gem_equip");  //&OFS_GEM_EQUIPMENT);
		hGemAdapt = oapiLoadMeshGlobal("gem_adapt");  //&OFS_GEM_ADAPTER);
		hGemCapsule = oapiLoadMeshGlobal("gem_capsule");  //&OFS_GEM_CAPSULE);
		hGemHeatshield = oapiLoadMeshGlobal("gem_heatshield1");  //&OFS_GEM_HEATSHIELD);
		hGemCdrSeat = oapiLoadMeshGlobal("gem-seat");  //&OFS_GEM_CDR);
		hGemPltSeat = oapiLoadMeshGlobal("gem-ejseat");  //&OFS_SEAT);
		hGemDoorL = oapiLoadMeshGlobal("gem_door_l");  //&OFS_GEM_DOOR_LEFT);
		hGemDoorR = oapiLoadMeshGlobal("gem_door_r");  //&OFS_GEM_DOOR_RIGHT);
		hGemNose = oapiLoadMeshGlobal("gem_nose");  //&OFS_GEM_NOSE);
	}
	if (fairing == 1) fairing = 301;
	if (fairing == 2) fairing = 501;	// Backwards compatibility for CVELTitans v1.1 scenarios
	if (fairing == 3) fairing = 601;


/* Titan fairings:

				501a
			   / || \
	  Size (dia) ||  a/b = left/right for 2-part fairings, a/b/c for 3-part
	0 = none	     | \
	1 = 5'		 |   Length
	2 = 8'		  \
	3 = 10'		    0 = 2-part fairing, 1 = 3-part
	4 = 11.5'?
	5 = 4m (13.1')
	6 = 5.08m (16.7')

  Special exception fairings:
 -100 = Launch escape system?
    0 = No fairing, of course
  300 = Titan 3C nose cone
  500 = 2-part fairing meant to be used on dual-payload Titan III (it's 4m with no neck-down at the bottom)
			Use 2PLCarr and 2PLPAF meshes with it.
			Also works well on Titan-C

*/

	if (fairing == 101) {
		hshroud1 = oapiLoadMeshGlobal ("CVELTitan/Fairing101b");
		hshroud2 = oapiLoadMeshGlobal ("CVELTitan/Fairing101a");
		hshroud3 = oapiLoadMeshGlobal ("CVELTitan/dummy");
		mass_fairing = 100.0;
	}
	else if (fairing == 102) {
		hshroud1 = oapiLoadMeshGlobal ("CVELTitan/Fairing102b");
		hshroud2 = oapiLoadMeshGlobal ("CVELTitan/Fairing102a");
		hshroud3 = oapiLoadMeshGlobal ("CVELTitan/dummy");
		mass_fairing = 120.0;
	}
	else if (fairing == 201) {
		hshroud1 = oapiLoadMeshGlobal ("CVELTitan/Fairing201b");
		hshroud2 = oapiLoadMeshGlobal ("CVELTitan/Fairing201a");
		hshroud3 = oapiLoadMeshGlobal ("CVELTitan/dummy");
		mass_fairing = 150.0;
	}
	else if ((fairing == 0) || (fairing == 300)) {					// Zero means no fairing.  But I've still got to
		hshroud1 = oapiLoadMeshGlobal ("CVELTitan/Fairing300b");	// put SOMETHING in hshroud1, 2, and 3
		hshroud2 = oapiLoadMeshGlobal ("CVELTitan/Fairing300a");
		hshroud3 = oapiLoadMeshGlobal ("CVELTitan/dummy");
		if (fairing == 300) mass_fairing = 150.0;
	}
	else if (fairing == 301) {
		hshroud1 = oapiLoadMeshGlobal ("CVELTitan/Fairing301b");
		hshroud2 = oapiLoadMeshGlobal ("CVELTitan/Fairing301a");
		hshroud3 = oapiLoadMeshGlobal ("CVELTitan/dummy");
		mass_fairing = 500.0;
	}
	else if (fairing == 311) {
		hshroud1 = oapiLoadMeshGlobal ("CVELTitan/Fairing311b");
		hshroud2 = oapiLoadMeshGlobal ("CVELTitan/Fairing311a");
		hshroud3 = oapiLoadMeshGlobal ("CVELTitan/Fairing311c");
		mass_fairing = 652.0;
	}
	else if (fairing == 312) {
		hshroud1 = oapiLoadMeshGlobal ("CVELTitan/Fairing312b");
		hshroud2 = oapiLoadMeshGlobal ("CVELTitan/Fairing312a");
		hshroud3 = oapiLoadMeshGlobal ("CVELTitan/Fairing312c");
		mass_fairing = 750.0;
	}
	else if (fairing == 313) {
		hshroud1 = oapiLoadMeshGlobal ("CVELTitan/Fairing313b");
		hshroud2 = oapiLoadMeshGlobal ("CVELTitan/Fairing313a");
		hshroud3 = oapiLoadMeshGlobal ("CVELTitan/Fairing313c");
		mass_fairing = 909.0;
	}
	else if (fairing == 314) {
		hshroud1 = oapiLoadMeshGlobal ("CVELTitan/Fairing314b");
		hshroud2 = oapiLoadMeshGlobal ("CVELTitan/Fairing314a");
		hshroud3 = oapiLoadMeshGlobal ("CVELTitan/Fairing314c");
		mass_fairing = 1075.0;
	}
	else if (fairing == 401) {
		hshroud1 = oapiLoadMeshGlobal ("CVELTitan/Fairing401b");
		hshroud2 = oapiLoadMeshGlobal ("CVELTitan/Fairing401a");
		hshroud3 = oapiLoadMeshGlobal ("CVELTitan/dummy");
		mass_fairing = 750.0;
	}
	else if (fairing == 402) {
		hshroud1 = oapiLoadMeshGlobal ("CVELTitan/Fairing402b");
		hshroud2 = oapiLoadMeshGlobal ("CVELTitan/Fairing402a");
		hshroud3 = oapiLoadMeshGlobal ("CVELTitan/dummy");
		mass_fairing = 900.0;
	}
	else if (fairing == 500) {
		hshroud1 = oapiLoadMeshGlobal ("CVELTitan/Fairing500b");
		hshroud2 = oapiLoadMeshGlobal ("CVELTitan/Fairing500a");
		hshroud3 = oapiLoadMeshGlobal ("CVELTitan/dummy");
		mass_fairing = 1875.0;										// Make sure 2PLPAF and 2PLCarr are 500 each
	}
	else if (fairing == 501) {
		hshroud1 = oapiLoadMeshGlobal ("CVELTitan/Fairing501b");
		hshroud2 = oapiLoadMeshGlobal ("CVELTitan/Fairing501a");
		hshroud3 = oapiLoadMeshGlobal ("CVELTitan/dummy");
		mass_fairing = 1000.0;
	}
	else if (fairing == 611) {
		hshroud1 = oapiLoadMeshGlobal ("CVELTitan/Fairing611b");
		hshroud2 = oapiLoadMeshGlobal ("CVELTitan/Fairing611a");
		hshroud3 = oapiLoadMeshGlobal ("CVELTitan/Fairing611c");
		mass_fairing = 3600.0;
	}
	else if (fairing == 601) {
		hshroud1 = oapiLoadMeshGlobal ("CVELTitan/Fairing601b");
		hshroud2 = oapiLoadMeshGlobal ("CVELTitan/Fairing601a");
		hshroud3 = oapiLoadMeshGlobal ("CVELTitan/dummy");
		mass_fairing = 1500.0;
	}
	else if (fairing == 611) {
		hshroud1 = oapiLoadMeshGlobal ("CVELTitan/Fairing611b");
		hshroud2 = oapiLoadMeshGlobal ("CVELTitan/Fairing611a");
		hshroud3 = oapiLoadMeshGlobal ("CVELTitan/Fairing611c");
		mass_fairing = 3600.0;
	}
	else if (fairing == 612) {
		hshroud1 = oapiLoadMeshGlobal ("CVELTitan/Fairing612b");
		hshroud2 = oapiLoadMeshGlobal ("CVELTitan/Fairing612a");
		hshroud3 = oapiLoadMeshGlobal ("CVELTitan/Fairing612c");
		mass_fairing = 5000.0;
	}
	else if (fairing == 613) {
		hshroud1 = oapiLoadMeshGlobal ("CVELTitan/Fairing613b");
		hshroud2 = oapiLoadMeshGlobal ("CVELTitan/Fairing613a");
		hshroud3 = oapiLoadMeshGlobal ("CVELTitan/Fairing613c");
		mass_fairing = 5500.0;
	}
	else if (fairing == 614) {
		hshroud1 = oapiLoadMeshGlobal ("CVELTitan/Fairing614b");
		hshroud2 = oapiLoadMeshGlobal ("CVELTitan/Fairing614a");
		hshroud3 = oapiLoadMeshGlobal ("CVELTitan/Fairing614c");
		mass_fairing = 5900.0;
	}
	else if (fairing == 615) {
		hshroud1 = oapiLoadMeshGlobal ("CVELTitan/Fairing615b");
		hshroud2 = oapiLoadMeshGlobal ("CVELTitan/Fairing615a");
		hshroud3 = oapiLoadMeshGlobal ("CVELTitan/Fairing615c");
		mass_fairing = 6300.0;
	}

	if (numSRMs1 == 10) {
		numSRMs0 = 6;
		numSRMs1 = 4;
	}
	if ((numSRMs1 != 4) && (numSRMs1 != 2))
		numSRMs1 = 0;

	if (status == 0) {
		if (SRM_Type == RSRM) {
			OFFSIDEL = RSRM_OFS_LEFT;
			OFFSIDER = RSRM_OFS_RIGHT;
			OFFSIDEL.x *= 1.1;
			OFFSIDEL.y *= 1.05;
			OFFSIDER.x *= 1.1;
			OFFSIDER.y *= 1.05;
		}
		else {
			OFFSIDEL = STAGE0_OFS_LEFT;
			OFFSIDER = STAGE0_OFS_RIGHT;
		}
		OFFSIDEL.z = 0.0;
		OFFSIDER.z = 0.0;
	}
	else {
		OFFSIDEL = _V(0,0,0);
		OFFSIDER = _V(0,0,0);
	}


	invoked = false;


	if ((status == 1) && (fairing == 0)) status = 2;
	switch (status) {
	case 0:
		stage=0;
		SetSRMsStage();
		break;
	case 1:
		stage=1;
		SetMainStage();
		break;
	case 2:
		stage=2;
		SetNoFairStage();
		break;
	case 3:
		stage = 3;
		status = 2;
		if (UprS == 0) deploy_no_more = true;
		SetSecondStage();
		break;
	case 4:
		stage = 4;
		status = 4;
		vectorme = false;
		SetUpperStage();
		break;
	case 13:
		stage = 3;
		status = 2;
		if (UprS == 0) deploy_no_more = true;
		ph_second = CreatePropellantResource (0.0);
		SetSecondStage();
	}

}




// ==============================================================
// API interface
// ==============================================================





/* basic vessel class setup */

void CVELTITAN::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetAStage();
}

void CVELTITAN::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	LoadState(  scn, vs );
}

void CVELTITAN::clbkSaveState (FILEHANDLE scn)
{
	SaveState(  scn );
}

void CVELTITAN::clbkPostStep (double simt, double simdt, double mjd)
{
	TimeStep(simt);
}

void CVELTITAN::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	SetAStage();
}

void CVELTITAN::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	SetAStage();
}

int CVELTITAN::clbkConsumeBufferedKey (DWORD dwKey, bool fPressed, char *keystate)
{
	int ret = ConsumeBufferedKey( dwKey, fPressed, keystate );
	return ret;
}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new CVELTITAN (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (CVELTITAN*)vessel;
}

